unit AgilentU;

interface

uses Windows, SysUtils, Forms, Dialogs, Libusb, Classes, ComCtrls, math,
     ActiveX, VisaComLib_TLB, jmmUtil;

const
   MaxVecteurDSO3000  = 4096;
   MaxVecteurDSO1000 = 600;
   MaxVecteurMemory = 10000;
   MaxVecteurDSO = 65535; // DSOX2000 100 000 / 2 = 50 000 par voie
   voie1 = 1;
   voie2 = 2;
   voie3 = 3;
   voie4 = 4;
   agilentVID = $0400; // Agilent DSO3000  avec libusb
   agilentPID = $c55d;
   GBFPID = 0;
   NombreMax = 1e37; // 9.9E+37 ou 99E+36
   VeffMin = 0.02;
   MemoryDepthUneVoie = 12000;


//VID 1008        |Hewlett Packard
//VID 2391  x0957 |Agilent Technologies, Inc.

   rigolVID = $0400; // 1024
   rigolPID = $05DC; // 1500
   rigol1054VID = $1AB1; // 6833
   rigol1054PID = $04CE; // 1230
   crCR       = #13; // $0D
   crLF       = #10; // $0A

// Agilent DSO 1000 avec visaCom  PID
//usb0::0x0957::0x0588::CN49251905::INSTR

// Agilent DSOX 2002 avec visaCom  PID
//usb0::0x0957::0x179B::MY54233244::INSTR

  DSOX2VID = $0957; // 2391
  DSOX2PID = $179B; // 6043

//GBF 33220  PID 1031
//usb0::0x0957::0x0407::MY43005238::INSTR

type
   pseudoWord = record case byte of
         0 : (octets : array[0..1] of ansiChar);
         1 : (mot : word);
         2 : (entier : smallInt);
         3 : (bits : array[0..1] of byte);
  end;
  indiceVecteurOsc  = 0..MaxVecteurDSO;
  vecteurOscInt   = array[indiceVecteurOsc] of integer;
  vecteurOscReal   = array[indiceVecteurOsc] of double;
  indiceVoieOsc = voie1..Voie2;
  TLectureWav = (wScreen,wMemory,wMaxPeak,wMinPeak);
  TEtatAttente = (atDiese, atNbreChiffre, atNbreData, atData, atEnd);
  Terreur = (erNo,erLow,erHigh,erHole);
  TTimeMode = (tmMain,tmWindow,tmXY,tmRoll);

  TvoieOsc = class
      private
      valeurMoy,valeurMin,valeurMax : integer;
      public
      Nom: string;
      active : boolean;
      nomUnite, Signif:  string;
      Origine,Increment,Reference: double;
      valeurInt :  vecteurOscInt;
   end;

   TvoieTemps = class
      private
      public
      Nom: string;
      nomUnite, Signif:  string;
      Origine,Increment,Reference: double;
   end;

   TDataAgilent = class
       private
       function GetValeur(voie : indiceVoieOsc;i : Integer): double;
       public
       debut,fin : indiceVecteurOsc;
       voies : array[indiceVoieOsc] of TvoieOsc;
       NbrePoints: integer;
       codage : integer;
       infiniVision : boolean;
       RigolDSZ : boolean;
       constructor create;
       property valeurs[voie : indiceVoieOsc;i: Integer] : double read GetValeur;
       function isOK(z : integer) : Terreur;
  end;

  TAgilent = class(TLIBUSB)
       private
       FVeff : double;
       Ffreq : double;
       Function GetPhase : double;
       Function GetDelay : double;
       procedure preambule(voie : integer);
       Function GetTimeMode : tTimeMode;
       procedure dump(voie : integer;lectureWav : TlectureWav);
       procedure WaveFormDataByte(voie : integer;lectureWav : TlectureWav);
       procedure WaveFormDataAscii(voie : integer;lectureWav : TlectureWav);
       function GetValeur(voie : indiceVoieOsc;i : Integer): double;
       function GetValeurTemps(i : Integer): double;
       function GetHole(i : Integer): boolean;
       function GetClipped : boolean;
       function getMemoryDepth(NbreVoies : integer) : integer;
       procedure setFrequence(avalue : double);
       procedure setVeff(avalue : double);
       public
       debut,fin : indiceVecteurOsc;
       voies : array[indiceVoieOsc] of TvoieOsc;
       voieT : TvoieTemps;
       NbrePoints: integer;
       ProgressBar : TprogressBar;
       stop : boolean; //  positionner dans le programme principal pour dbloquer la lecture
       infiniVision : boolean;
       rigolDSZ : boolean;
       avgCount : integer;
       codage : integer;
       pbDataByte : boolean;
       VeffMax : double;
       constructor Create;
       destructor Destroy; override;
       property TimeMode : TTimeMode read getTimeMode;
       property frequence : double read FFreq write setFrequence;
       property clipped : boolean read getClipped;
       property Veff : double read FVeff write setVeff;
       property hole[i: Integer] : boolean read GetHole;
       property valeursT[i: Integer] : double read GetValeurTemps;
       property valeurs[voie : indiceVoieOsc;i: Integer] : double read GetValeur;
       property phase : double read getPhase;
       property delay : double read getDelay;
       procedure unLock; // dbloque les touches de l'oscillo
       procedure litValeurs(lectureWav : TlectureWav); // le rsultat est dans valeurs
       procedure litValeursInfiniVision;
       procedure litValeursRigolDSZ;
       procedure litValeursRigolDSZbis;
       function getCoeffWav(command: AnsiString;defaut : double) : double;
       procedure KeyCommand(command: AnsiString);
       procedure RootCommand(command: AnsiString);
       procedure TrigCommand(command: AnsiString);
       procedure ChanCommand(command: AnsiString;ch : integer);
       function getFloat(command: AnsiString;defaut : double) : double;
       Function getFloatUnite(command: ansiString) : double;
       function getInteger(command: AnsiString;defaut : integer) : integer;
       function getMeasure(command: AnsiString;voie : integer) : double;
       function getRMSAC(voie : integer) : double;
       function getVoieActive(voie : integer) : boolean;
       function getBool(ordre : AnsiString) : boolean;
       function setChanScaleEff(Veff : double;ch : integer) : double;
       function setChanScaleCalibre(gamme : double;ch : integer) : double;
  //     procedure setPhase12;
       procedure setTimeScale(freq : double);
       procedure waitOPC;
       procedure init(anom : string);
       procedure ChercheUnTour(demiTour: boolean);
       procedure sauve(var data : TdataAgilent);
       function valeurVolt(voie : indiceVoieOsc;valeurI : integer) : double;
       function zero(voie : indiceVoieOsc) : integer;
       function isOK(z : integer) : Terreur;
       procedure sinus(init : boolean);
  end;

  TGBF = class(TLIBUSB)
       private
       FVeff : double;
       Ffreq : double;
       procedure setFrequence(avalue : double);
       procedure setVeff(avalue : double);
       public
       VeffMax : double;
       property frequence : double read FFreq write setFrequence;
       property Veff : double read FVeff write setVeff;
       constructor Create;
       destructor Destroy; override;
       procedure unLock; // dbloque les touches du GBF
       procedure Lock; // dbloque les touches du GBF
       procedure waitOPC;
       function getString(command: AnsiString) : AnsiString;
       procedure init(anom : string);
       procedure sinus(init : boolean);
  end;

var
    agilent : TAgilent;
    GBF : TGBF;

implementation

function StrToFloatA(const nombre: ansiString): double;
var
   posSeparator,posBlanc: integer;
   N : string;
begin
   N := string(nombre);
   if formatSettings.decimalSeparator=',' then begin
       posSeparator := pos('.', N);
       if posSeparator > 0 then
          N[posSeparator] := ',';
   end;
   posBlanc := pos(' ',N);
   while posBlanc>0 do begin
       delete(N,posBlanc,1);
       posBlanc := pos(' ',N);
   end;
   try
      result := strToFloat(N);
   except
      result := Math.Nan
   end;
end;

function StrToFloatAgilent(nombre: string): double;
var
   posSeparator: integer;
begin
   if formatSettings.decimalSeparator=',' then begin
      posSeparator := pos('.', nombre);
      if posSeparator > 0 then
         Nombre[posSeparator] := ',';
   end;
   try
      result := strToFloat(nombre);
   except
      result := Math.Nan
   end;
end;


function Atan2(im, re: double): double;
// z = re + j*im
begin
   try
      Result := arcTan(im / re);
      if re < 0 then
         if im < 0 then
            Result := Result - pi
         else
            Result := Result + pi;
   except
      on EZeroDivide do begin
         if im < 0 then
            Result := -pi / 2
         else
            Result := pi / 2;
      end;
   end;
   Result := RadToDeg(Result);
end;

function TAgilent.getCoeffWav(command: AnsiString;defaut : double) : double;
begin
      result := getFloat(':WAV:'+command+'?',defaut);
end;

procedure TAgilent.keyCommand(command: AnsiString);
begin
      if not infiniVision then begin
         writeString(':KEY:'+command);
         waitOPC;
      end;
end;

procedure TAgilent.rootCommand(command: AnsiString);
begin
      writeString(':'+command);
      waitOPC;
end;

procedure TAgilent.unLock;
begin
      if usb=oRigol
         then keyCommand('FORCE')
         else if infiniVision
            then writeString(':SYST:LOCK OFF')
            else if RigolDSZ
               then keyCommand('FORCE')
               else writeString(':KEY:TrigFORCe')
end;

procedure TGBF.unLock;
begin
    writeString(':SYST:KLOC OFF')
end;

procedure TGBF.Lock;
begin
    writeString(':SYST:KLOC ON')
end;

procedure TAgilent.trigCommand(command: AnsiString);
begin
      if usb=oVisaCOM then
         command := 'EDGE:'+command;
      writeString(':TRIG:'+command);
      waitOPC;
end;

procedure TAgilent.chanCommand(command: AnsiString;ch : integer);
var st : ansiString; // pour debug
begin
      st := ':CHAN'+AnsiString(intToStr(ch))+':'+command;
      writeString(st);
      waitOPC;
end;

function TAgilent.getFloat(command: AnsiString;defaut : double) : double;
var reponse : ansiString;
begin
      result := defaut;
      reponse := getString(command);
      if reponse='' then exit;
      try
      if reponse[length(reponse)]='%' then // duty cycle
         reponse := copy(reponse,1,length(reponse)-1);
      if reponse[1]='<'
         then result := 0 // phase ! ?
         else begin // 9.9E+37 ou 99E+36
            result := StrToFloatA(reponse);
            if result>NombreMax  then result := Math.Nan;
         end;
      except
      end;
end;

Function TAgilent.GetFloatUnite(command: ansiString) : double;
var reponse : ansiString;
    coeff : AnsiChar;
begin
     result := nan;
     try
     reponse := GetString(command);
     if reponse='' then exit;
     if reponse[length(reponse)]='z'
        then reponse := copy(reponse,1,length(reponse)-2) // Hz
        else reponse := copy(reponse,1,length(reponse)-1); // V ou s
     coeff := reponse[length(reponse)];
     if not (coeff in ['0'..'9'])
         then reponse := copy(reponse,1,length(reponse)-1);
     result := StrToFloatWin(string(reponse));
     case coeff of
         'u' : result := result*1E-6;
         'm' : result := result*1E-3;
         'n' : result := result*1E-9;
         'k' : result := result*1E3;
     end;
     if abs(result)>9e37 then result := Nan;
     except
     end;
end;

function TAgilent.getInteger(command: AnsiString;defaut : integer) : integer;
var reponse : string;
begin
      result := defaut;
      reponse := string(getString(command));
      try
      result := StrToInt(reponse);
      except
      end;
end;

function TAgilent.getMeasure(command: AnsiString;voie : integer) : double;
begin
      if RigolDSZ
         then result := getFloat(':MEAS:ITEM? '+command+',CHAN'+AnsiString(intToStr(voie)),0)
         else result := getFloat(':MEAS:'+command+'? CHAN'+AnsiString(intToStr(voie)),0);
end;

function TAgilent.getRMSAC(voie : integer) : double;
begin
     if RigolDSZ
         then result := getFloat(':MEAS:ITEM? VRMS,CHAN'+AnsiString(intToStr(voie)),0)
         else result := getFloat(':MEAS:VRMS?AC,CHAN'+AnsiString(intToStr(voie)),0);
end;

function TAgilent.getVoieActive(voie : integer) : boolean;
begin
      result := getBool(':CHAN'+AnsiString(intToStr(voie))+':DISP?');
end;

function TAgilent.getBool(ordre : ansiString) : boolean;
var reponse : ansiString;
begin
      result := false;
      if not open then exit;
      reponse := getString(ordre);
      result := (reponse='1') or (reponse='ON');
end;

function TGBF.getString(command : AnsiString) : AnsiString;
var len : integer;
    buf : ansiString;
begin
      result := '';
      writeString(command);
      len := readString(buf);
      if (len>0) and (buf[len]=crLF) then buf := copy(buf,1,len-1);
      result := buf;
end;

procedure TAgilent.dump(voie : integer;lectureWav : TlectureWav);
var buffer,tampon : AnsiString;
    i,j,decalage : integer;
    enCours : boolean;
begin
  if usb=oRigol
  then writeString(':WAV:DATA? CHAN'+AnsiString(intToStr(voie)))
  else writeString(':WAV:SOUR CHAN'+AnsiString(intToStr(voie)));
  waitOPC;
  case usb of
     oLibUsb : voies[voie].reference := 125; // DSO 3000
     oRigol : voies[voie].reference := 125;
     oVisaCom : voies[voie].reference := getCoeffWav('YREF',0); // DSO 1000
  end;
  voies[voie].origine := getCoeffWav('YOR',0);
  voies[voie].increment := getCoeffWav('YINC',0.08);
//   if usb=oRigol then voies[voie].increment := getCoeffWav(':CHAN'+intoStr(voie)+':OFFS?);
  case lectureWav of
     wScreen : tampon := 'DATA';
     wMemory : tampon := 'MEM';
     wMaxPeak : tampon := 'MAXP';
     wMinPeak : tampon := 'MINP';
  end;
  writeString(':WAV:'+tampon+'?');
  tampon := '';
  j := 0;
  decalage := maxVecteurDSO3000*(voie-voie1);
  enCours := true;
  while readString(buffer)>0 do begin
      if enCours
      then for i := 1 to length(buffer) do
          case buffer[i] of
             ' ' : if enCours then begin
                  tampon := copy(tampon,3,2); // 0x12
                  if length(tampon)=2 then begin
                     try
                     voies[voie].valeurInt[j] := StrToInt('$' + string(tampon)) ;
                     inc(j);
                     if (progressBar<>nil) and
                        ( (j mod 32)=0 ) then progressBar.Position := decalage+j;
                     except
                        showMessage('Erreur lecture');
                     end;
                  end;
                  tampon := '';
              end;
              crLF,crCR : enCours := false; // il y a des donnes ensuite  ne pas prendre en compte ?
              else if enCours then tampon := tampon + buffer[i];
          end // case
      else sleep(10);
      if stop then break;
  end;
  NbrePoints := j;
  debut := 0;
  fin := pred(j);
  if not stop then
     progressBar.Position := decalage+maxVecteurDSO3000;
end;

procedure TAgilent.preambule(voie : integer);
var posVal:TStringList;

    reponse : string;
begin

   reponse := string(GetString('WAV:PRE?'));

   posVal := Tstringlist.Create;
   posVal.Delimiter := ',';
   posVal.DelimitedText := reponse;
//   intFormat = Preamble(0)
//   intType = Preamble(1)
   NbrePoints := strToInt(posVal[2]);
   debut := 0;
   fin := pred(NbrePoints);
   AvgCount := strToInt(posVal[3]);
   with voieT do begin
   increment := abs(strToFloatPoint(posval[4])); // abs ??
   Origine := strToFloatPoint(posval[5]);
   if origine>0 then origine := 0;
   Reference := strToFloatPoint(posval[6]);
   end;
   with voies[voie] do begin
   increment := strToFloatPoint(posVal[7]);
   Origine := strToFloatPoint(posVal[8]);
   Reference := strToFloatPoint(posVal[9]);
   end;
   posVal.Free;
end;

procedure TAgilent.WaveFormDataByte(voie : integer;lectureWav : TlectureWav);
var i,decalage : integer;
    Attente: TetatAttente;
    nbreChiffre : integer;
    nbreCourant : integer;
    ChaineNbreData : AnsiString;
    nbreMax : integer;
    buffer : ansiString;
    longReponse : longInt;
    zint : longInt;
begin
  if lectureWav=wScreen
     then NbreMax := MaxVecteurDSO1000
     else NbreMax := MaxVecteurMemory;
  writeString(':WAV:SOUR CHAN'+AnsiString(intToStr(voie)));
  waitOPC;
  writeString(':WAV:FORM BYTE');
  codage := varByte;
  preambule(voie);
  writeString(':WAV:DATA?');
  sleep(1000);
  decalage := NbreMax*(voie-voie1);
  Attente := atDiese;
  nbreCourant := 0;
  nbreChiffre := 8;
  longReponse := ReadString(buffer);
// S_OK 0
// S_VISA_MAX_CNT 0x40006 : on a lu 1024 caractres
// S_VISA_TERM_CHAR 0x40005 : fin des donnes
// <0 erreur
  while (longReponse>0) and
        (attente<>atEnd) and not stop do begin
        for i := 1 to length(buffer) do begin
         case Attente of
            atEnd: ;
            atDiese: if buffer[i] = '#'
                  then Attente := atNbreChiffre;
            atNbreChiffre: begin
               NbreChiffre := strToInt(buffer[i]); // 8
               Attente := AtNbreData;
               ChaineNbreData := '';
            end;
            atNbreData: begin
               ChaineNbreData := ChaineNbreData + buffer[i];
               if length(ChaineNbreData) = NbreChiffre then begin
                  Attente := atData;
                  nbreMax := StrToInt(string(ChaineNbreData));
               end;
            end;
            atData: begin
                  try
                     zint := ord(buffer[i]);
                     if zint>255 then zint := 255;
                  except
                     zint := 0;
                  end;
                  voies[voie].valeurInt[NbreCourant] := zint;
                  Inc(NbreCourant);
                  if (progressBar<>nil) and
                     ( (NbreCourant mod 32)=0 ) then begin
                        progressBar.Position := decalage+NbreCourant;
                        application.ProcessMessages;
                  end;
                  if NbreCourant >= nbreMax then attente := atEnd;
               end;
         end; // case
      end;// for i
      if reponseVisa=S_VISA_TERM_CHAR
         then attente := atEnd
         else if attente<>atEnd then begin
            longReponse := ReadString(buffer);
            application.ProcessMessages;
         end;
  end;
  NbrePoints := NbreCourant;
  debut := 0;
  fin := pred(NbreCourant);
  progressBar.Position := decalage+nbreCourant;
  PbDataByte := nbreCourant<(nbreMax div 2);
end;


procedure TAgilent.WaveFormDataAscii(voie : integer;lectureWav : TlectureWav);

var i,decalage : integer;

    nbreCourant : integer;
    nbreMax : integer;
    buffer : ansiString;
    longReponse : longInt;
    reponse : AnsiString;
    reponseL : TstringList;
    sauveVirgule : char;
begin
  reponseL := TstringList.Create;
  reponseL.delimiter := ',';
  if lectureWav=wScreen
     then NbreMax := MaxVecteurDSO1000
     else NbreMax := MaxVecteurMemory;
  writeString(':WAV:SOUR CHAN'+AnsiString(intToStr(voie)));
  waitOPC;
  writeString(':WAV:FORM ASCII');
  codage := varByte;
  preambule(voie);
  writeString(':WAV:DATA?');
  sleep(1000);
  decalage := NbreMax*(voie-voie1);
  reponse := '';
  repeat
      longReponse := ReadString(buffer);
      reponse := reponse+buffer;
      nbreCourant := length(reponse) div 12;
      if (progressBar<>nil) and
         (NbreCourant>progressBar.Position+32) then begin
            progressBar.Position := decalage+NbreCourant;
            progressBar.Update;
            application.ProcessMessages;
      end;
      application.ProcessMessages;  // pour stop
  until (longReponse<1000) or stop;
  reponseL.DelimitedText := string(reponse);
  nbreCourant := reponseL.count-1;
  progressBar.Position := decalage+NbreCourant;
  sauveVirgule := formatSettings.DecimalSeparator;
  formatSettings.DecimalSeparator := '.';
  with voies[voie] do begin
     origine := 0;
     increment := -increment;
     for i := 1 to reponseL.count-1 do
         valeurInt[i-1] := round(StrToFloat(reponseL[i])/Increment)+round(reference);
  end;
  formatSettings.DecimalSeparator := sauveVirgule;
  NbrePoints := NbreCourant;
  debut := 0;
  fin := pred(NbreCourant);
  progressBar.Position := decalage+nbreCourant;
  reponseL.Free;
end;

procedure TAgilent.litValeursRigolDSZ;
var Nombre : array[voie1..voie4] of integer;
    NbreRigol : integer;

procedure litVoie(voie : integer);
var i,j : integer;
    nbreChiffre : integer;
    ChaineNbreData : string;
    buffer : CharArray;
    count : longInt;
    NbreLu : integer;
begin
  try
  writeString(':WAV:SOUR CHAN'+AnsiString(intToStr(voie)));
  waitOPC;
  voies[voie].reference := getInteger(':WAV:YREF?',127);  // always 127
  voies[voie].origine := getInteger(':WAV:YOR?',0);
  voies[voie].increment := getFloat(':WAV:YINC?',0.08);
  writeString(':WAV:STAR 1');
  writeString(':WAV:STOP '+AnsiString(intToStr(NbreRigol)));
  writeString(':WAV:DATA?');
  sleep(1000);
  count := NbreRigol+11; // #9xxxxxxxxx
  setLength(buffer,count);
  reponseVisa := ReadWord(count,buffer);
  // buffer[0] = '#'
  NbreChiffre := ord(buffer[1])-ord('0'); // 9
  for i := 2 to 2+NbreChiffre-1 do
      ChaineNbreData := ChaineNbreData + char(buffer[i]);
  i := 2+NbreChiffre;
  nbreLu := StrToInt(ChaineNbreData);
  for j := 0 to pred(NbreLu) do begin
      voies[voie].valeurInt[j] := buffer[i];
      Inc(i);
  end;
  Nombre[voie] := NbreLu;
  except
     Nombre[voie] := 0;
  end;
end;

var j,voie : integer;
    NbreVoies : integer;
begin
  voieT.reference := 0;
  voieT.origine := 0;
  nbreVoies := 0;
  for voie := voie1 to high(indiceVoieOsc) do begin
      voies[voie].active := getVoieActive(voie);
      inc(NbreVoies);
  end;
  if NbreVoies=0  then exit;
  voieT.increment := getCoeffWav('XINC',1e-5);
  if NbreVoies=3 then NbreVoies := 4;
  NbreRigol := MemoryDepthUneVoie div NbreVoies;
  writeString(':WAV:MODE NORM');
  writeString(':WAV:FORM BYTE');
  codage := varByte;
  for voie := voie1 to high(indiceVoieOsc) do begin
         if voies[voie].active then begin
            LitVoie(voie);
            if NbrePoints=0
               then NbrePoints := Nombre[voie]
               else if NbrePoints>Nombre[voie]
                    then NbrePoints := Nombre[voie]
         end
         else for j := 0 to NbreRigol do
              voies[voie].valeurInt[j] := 255;
     end;
  debut := 0;
  fin := pred(NbrePoints);
end;

procedure TAgilent.litValeursRigolDSZBis;
var Nombre : array[voie1..voie4] of integer;
    NbreRigol : integer;

procedure litVoie(voie : integer;pas : integer);
var i,j,k : integer;
    nbreChiffre : integer;
    ChaineNbreData : string;
    buffer : CharArray;
    count : longInt;
    iMax : integer;
begin
  try
  writeString(':WAV:SOUR CHAN'+AnsiString(intToStr(voie)));
  waitOPC;
  voies[voie].reference := getCoeffWav('YREF',0);  // always 127
  voies[voie].origine := getCoeffWav('YOR',0);
  voies[voie].increment := getCoeffWav('YINC',0.08);
  j := 0;
  progressBar.position := 0;
  progressBar.Invalidate;
  for k := 0 to pas-1 do begin
      writeString(':WAV:STAR '+AnsiString(intToStr(1+k*NbreRigol)));
      writeString(':WAV:STOP '+AnsiString(intToStr((k+1)*NbreRigol)));
      writeString(':WAV:DATA?');
      sleep(1000);
      count := NbreRigol+11; // #9xxxxxxxxx
      setLength(buffer,count);
      reponseVisa := ReadWord(count,buffer);
      // buffer[0] = '#'
      NbreChiffre := ord(buffer[1])-ord('0'); // 8 ou 9
      for i := 2 to 2+NbreChiffre-1 do
          ChaineNbreData := ChaineNbreData + char(buffer[i]);
      i := 2+NbreChiffre;
      iMax := i+StrToInt(ChaineNbreData);
      while (i<iMax) do begin
          voies[voie].valeurInt[j] := buffer[i];
          inc(j);
          Inc(i,pas);
      end;
      progressBar.position := k;
      progressBar.Invalidate;
  end;
  Nombre[voie] := j-1;
  except
     Nombre[voie] := 0;
  end;
end;

var voie : integer;
    NbreVoies : integer;
    pas : integer;
begin
  voieT.reference := 0;
  voieT.origine := 0;
  nbreVoies := 0;
  for voie := voie1 to high(indiceVoieOsc) do begin
      voies[voie].active := getVoieActive(voie);
      inc(NbreVoies);
  end;
  if NbreVoies=0  then exit;
  if NbreVoies=3 then NbreVoies := 4;
  voieT.increment := getCoeffWav('XINC',1e-5);
  NbreRigol := GetMemoryDepth(NbreVoies);
  pas := 1;
  while NbreRigol>MemoryDepthUneVoie do begin
        pas := pas * 10;
        NbreRigol := NbreRigol div 10;
  end;
  writeString(':WAV:MODE NORM');
  writeString(':WAV:FORM BYTE');
  codage := varByte;
  progressBar.Visible := true;
  progressBar.Max := pas;
  for voie := voie1 to high(indiceVoieOsc) do
         if voies[voie].active then begin
            LitVoie(voie,pas);
            if NbrePoints=0
               then NbrePoints := Nombre[voie]
               else if NbrePoints>Nombre[voie]
                    then NbrePoints := Nombre[voie]
         end;
  debut := 0;
  fin := pred(NbrePoints);
  progressBar.Visible := false;
end;



procedure TAgilent.litValeursInfiniVision;

var Nombre : array[voie1..voie2] of integer;


procedure EnTete;

begin
  writeString(':WAV:POIN:MODE NORM');
  if ProgressBar<>nil then ProgressBar.position := 0;
  writeString(':WAV:FORM WORD');
  codage := varWord;
  writeString(':WAV:BYT LSBF'); // comme GTS2 (sinon MSBF)
end;


procedure WaveForm(voie : integer);
var i : integer;
    nbreChiffre : integer;
    j : integer;
    ChaineNbreData : string;
    buffer : CharArray;
    motLu : pseudoWord;
    count : integer;
    NbreMax : integer;
begin
  try
  case voie of
      1 : writeString(':WAV:SOUR CHAN1');
      2 : writeString(':WAV:SOUR CHAN2');
      else exit;
  end;
  NbreMax := getInteger(':WAV:POIN?',1000);
  preambule(voie);
  writeString(':WAV:DATA?');
  count := 2*NbreMax+11; // #9xxxxxxxxx
  setLength(buffer,count);
  reponseVisa := ReadWord(count,buffer);
  // buffer[0] = '#'
  NbreChiffre := ord(buffer[1])-ord('0'); // 8 ou 9
  ChaineNbreData := '';
  for i := 2 to 2+NbreChiffre-1 do
      ChaineNbreData := ChaineNbreData + char(buffer[i]);
  i := 2+NbreChiffre;
  nbreMax := StrToInt(ChaineNbreData) div 2; // /2:word
  for j := 0 to pred(NbreMax) do begin
// 0000 = hole ; 0001 = clipped low ; FFFF = clipped high
      motLu.bits[0] := buffer[i];
      motLu.bits[1] := buffer[i+1];
      voies[voie].valeurInt[j] := motLu.mot;
      inc(i,2);
    end;// while
    (*
{$IFDEF DEBUG}
      append(fichierDebug);
      writeln(fichierDebug,'NbrePointsLu:',j);
      system.flush(fichierDebug);
      closeFile(fichierDebug);
{$ENDIF}
*)
    Nombre[voie] := NbreMax;
  except
    Nombre[voie] := 0;
  end;
end;

procedure setVoie(voie : integer);
var j : integer;
begin
      for j := 0 to MaxVecteurDSO do
          voies[voie].valeurInt[j] := 255
end;

var voie : indiceVoieOsc;
begin

     NbrePoints := 0;

     fin := 0;

     for voie := voie1 to voie2 do

         voies[voie].active := getVoieActive(voie);
     enTete;
     for voie := voie1 to voie2 do begin

         if voies[voie].active then begin
            WaveForm(voie);
            if NbrePoints=0
               then NbrePoints := Nombre[voie]
               else if NbrePoints>Nombre[voie]
                    then NbrePoints := Nombre[voie]
         end
         else setVoie(voie);
     end;
     debut := 0;
     fin := pred(NbrePoints);
end;


procedure TAgilent.litValeurs(lectureWav : TlectureWav);


procedure litValeursDSO3000;
var voie : indiceVoieOsc;

//   tm : TTimeMode;

begin

     if progressBar<>nil then begin

         if lectureWav=wMemory

              then ProgressBar.Max := 2*maxVecteurMemory

              else ProgressBar.Max := 2*maxVecteurDSO1000;

        application.ProcessMessages;

     end;

//     tm := timeMode;

     case lectureWav of

         wMemory : begin

            writeString(':WAV:POIN:MODE RAW');

            writeString(':WAV:POIN '+AnsiString(intToStr(MaxVecteurMemory)));

         end;

         wMaxPeak,wMinPeak : writeString(':WAV:POIN:MODE MAX');

         wScreen : writeString(':WAV:POIN:MODE NORM');

     end;

     NbrePoints := 0;

     fin := 0;

     if stop then exit;
     for voie := voie1 to voie2 do begin
         voies[voie].active := getVoieActive(voie);
         if voies[voie].active then begin
               WaveFormDataByte(voie,lectureWav);
               if pbDataByte then
                  WaveFormDataAscii(voie,lectureWav);
         end;
         if stop then exit;
     end;
end;

procedure litValeursLibUSB;
var voie : indiceVoieOsc;

begin

     if progressBar<>nil then begin

        ProgressBar.Max := 2*maxVecteurDSO3000;

        application.ProcessMessages;

     end;

     with voieT do begin

        reference := 0;

        origine := getCoeffWav('XOR',0);

        if origine>0 then origine := 0;

        increment := abs(getCoeffWav('XINC',1e-6)); // abs ??

     end;
     NbrePoints := 0;
     fin := 0;
     if stop then exit;
     for voie := voie1 to voie2 do begin
         voies[voie].active := getVoieActive(voie);
         if voies[voie].active then dump(voie,lectureWav);
         if stop then exit;
     end;
end;

begin
     stop := false;

     if progressBar<>nil then ProgressBar.position := 0;

     if usb=oLibUSB

        then litValeursLibUSB

        else if infiniVision

           then litValeursInfiniVision

           else if RigolDSZ

              then litValeursRigolDSZ

              else litValeursDSO3000;

end;


Procedure TAgilent.init(anom : string);

begin

  nomVisaCom := anom;

  open := true;

  if not open then exit;

  if (usb=oVisaCom) and

     (Pos('rigol',identificateur)>0) then usb := oRigol;
  if (usb=oVisaCom) and (usb<>oRigol) then begin
     infiniVision := pos('X',identificateur)>0;
// if not infiniVision then infiniVision := pos('MY54',NomVisaCom)>0;
// Agilent DSOX 2002
// usb0::0x0957::0x179B::MY54233244::INSTR
     rigolDSZ := (pos('RIGOL',identificateur)>0) and
                 (pos('DS1Z',identificateur)>0);
  end;
  if infiniVision then ; // ??  IO.Timeout = 10000
end;

Procedure TGBF.init(anom : string);
begin

  nomVisaCom := anom;

  open := true;

  if not open then exit;

  WriteString('*RST'); // Reset the function generator

  Sinus(true);

  IO.Clear; //Clear errors and status registers
end;

constructor TAgilent.create;
var voie : indiceVoieOsc;
begin
     inherited create(DSOX2VID,DSOX2PID);
     voieT := TvoieTemps.Create;
     for voie := voie1 to high(indiceVoieOsc) do begin
         voies[voie] := TvoieOsc.Create;
         voies[voie].nom := 'V'+intToStr(voie);
         voies[voie].nomUnite := 'V';
         voies[voie].Signif := 'Tension voie '+intToStr(voie);
     end;
     voieT.nom := 't';
     voieT.nomUnite := 's';
     voieT.Signif := 'temps';
     NbrePoints := 0;
     fin := 0;
     codage := varByte;
     VeffMax := 1.76;
end;

destructor TAgilent.destroy;
var voie : indiceVoieOsc;
begin
   for voie := voie1 to high(indiceVoieOsc) do
       voies[voie].free;
   inherited;
end;


constructor TDataAgilent.create;
var voie : indiceVoieOsc;
begin
     for voie := voie1 to high(indiceVoieOsc) do begin
         voies[voie] := TvoieOsc.Create;
         voies[voie].nom := 'V'+intToStr(voie);
         voies[voie].nomUnite := 'V';
         voies[voie].Signif := 'Tension voie '+intToStr(voie);
     end;
     NbrePoints := 0;
     fin := 0;
     codage := varByte;
end;

constructor TGBF.create;
begin
     inherited create(agilentVID,agilentPID);
     FVeff := 5;
     Ffreq := 1000;
     VeffMax := 7;
end;

destructor TGBF.destroy;

begin
     inherited
end;


Function TAgilent.GetPhase : double;

var indexZero1,indexZero2 : indiceVecteurOsc;
    periodique : boolean;

Procedure Calcul(v : indiceVoieOsc);
var i,zz : integer;
begin  with voies[v] do begin
    valeurMax := valeurInt[0];
    valeurMin := valeurInt[0];
    for i := 1 to pred(NbrePoints) do begin
        zz := valeurInt[i];
        if zz>valeurMax
           then valeurMax := zz
           else if zz<valeurMin
              then valeurMin := zz;
    end;
    valeurMoy := (valeurMax+valeurMin) div 2;
end end;

Procedure CherchePeriode(v : indiceVoieOsc);
var signe : double;
begin with voies[v] do begin
     periodique := true;
     indexZero1 := 0;
     repeat
        signe := valeurInt[indexZero1]-valeurMoy;
        inc(indexZero1);
     until (signe<>0) or (indexZero1>(NbrePoints div 2));
     while (indexZero1<(NbrePoints div 2)) and
           ((valeurInt[indexZero1]-valeurMoy)*signe>0) do
              inc(indexZero1);
// (valeurInt[indexZero1]-valeurMoy)*signe<=0
// Premier passage par "zro" (vers le bas par ex.)
     indexZero2 := indexZero1+16;
     while (indexZero2<(NbrePoints-16)) and
           ((valeurInt[indexZero2]-valeurMoy)*signe<0) do
           inc(indexZero2);
// (valeurInt[indexZero2]-valeurMoy)*signe>=0
// On repasse par "zro" dans le sens oppos
     indexZero2 := indexZero2+16;
     while (indexZero2<NbrePoints) and
          ((valeurInt[indexZero2]-valeurMoy)*signe>0) do
          inc(indexZero2);
// (valeurInt[indexZero2]-valeurMoy)*signe<=0
// On repasse par "zro" dans le mme sens
     if indexZero2>=NbrePoints then begin
        indexZero1 := 0;
        indexZero2 := pred(nbrePoints);
        periodique := false;
     end;
end end;

Function GetPhaseLibUsb : double;
var i,decalage : integer;
    somme,sommeQuartPeriode : double;
begin
    result := Math.Nan;
    if not voies[voie1].active or
       not voies[voie2].active then exit;
    calcul(voie1);calcul(voie2);
    if (voies[voie1].valeurMax-voies[voie1].valeurMin)>
       (voies[voie2].valeurMax-voies[voie2].valeurMin)
// on prend la plus prcise
      then cherchePeriode(voie1)
      else cherchePeriode(voie2);
    if not periodique then exit;
    decalage := (indexZero2-indexZero1) div 4; // un quart de priode
    if (indexZero2+decalage>=NbrePoints) then exit;
    Somme := 0;
    SommeQuartPeriode := 0;
    for i := indexZero1 to pred(indexZero2) do begin
          Somme := Somme+
                 (voies[voie1].valeurInt[i]-voies[voie1].valeurMoy)*
                 (voies[voie2].valeurInt[i]-voies[voie2].valeurMoy);
          SommeQuartPeriode := SommeQuartPeriode+
                 (voies[voie1].valeurInt[i]-voies[voie1].valeurMoy)*
                 (voies[voie2].valeurInt[i+decalage]-voies[voie2].valeurMoy);
    end;
    result := atan2(SommeQuartperiode,Somme);
end;

begin
     case usb of
          oVisaCom : if infiniVision
             then result := getFloat(':MEAS:PHAS? CHAN1,CHAN2',0)
             else if RigolDSZ
                  then result := getFloat(':MEAS:ITEM? RPH',0)
                  else result := getFloat(':MEAS:PPHA?',0);
          olibusb : result := getPhaseLibUsb;
          oRigol : result := getPhaseLibUsb;
          else result := Nan;
     end;
     if isNan(result) then exit;
     if result>180 then result := result-360;
     if result<-180 then result := result+360;
end;

Function TAgilent.GetDelay : double;
begin
     if (usb=oVisaCom) then
         if infiniVision
            then result := getFloat(':MEAS:DEL?',0)
            else result := getFloat(':MEAS:ITEM? RDEL',0)
         else result := 0
end;

Function TAgilent.GetTimeMode : TTimeMode;
var reponse : ansiString;
begin
     result := tmMain;
     if (usb=oVisaCom) and (infiniVision or RigolDSZ) then begin
            reponse := getString(':TIM:MODE?');
            if reponse='XY' then result := tmXY;
            if pos('WIND',string(reponse))>0 then result := tmWindow;
            if reponse='ROLL' then result := tmRoll;
     end
end;

(*
procedure TAgilent.setPhase12;
begin
    if infiniVision then
       writeString(':MEAS:PHAS CHAN1,CHAN2');
end;
*)

procedure TAgilent.ChercheUnTour(demiTour: boolean);

   const
      EcartMin = 0.05;
      EcartMax = 0.5;// min, max pour hystrsis
   var
      i, i0: indiceVecteurOsc;
      CoeffY, CoeffX: double;
      minLoc, ecartLoc: double;

      function GetCoeff(const V: VecteurOscInt): double;
      var
         i: indiceVecteurOsc;
         mini, maxi: double;
      begin
         Mini := V[1];
         Maxi := V[1];
         for i := 1 to pred(NbrePoints) do
            if V[i] > Maxi
               then Maxi := V[i]
               else if V[i] < Mini then
                   Mini := V[i];
         Result := 1 / (Maxi - Mini);
      end;

      function ecart: double;
      begin
         ecart := abs(CoeffY * (voies[voie1].valeurInt[i] - voies[voie1].valeurInt[i0])) +
                  abs(CoeffX * (voies[voie2].valeurInt[i] - voies[voie2].valeurInt[i0]));
      end;

      procedure ChercheIdebut;
      var
         xmin, xmax, ymin, ymax: integer;
         ixmin, ixmax, iymin, iymax: integer;
         i: integer;
      begin
         ixmin := 0;
         ixmax := 0;
         iymin := 0;
         iymax := 0;
         i0 := 0;
         xmin := voies[voie1].valeurInt[0];
         xmax := voies[voie1].valeurInt[0];
         ymin := voies[voie2].valeurInt[0];
         ymax := voies[voie2].valeurInt[0];
         for i := 1 to NbrePoints div 2 do begin
            if voies[voie1].valeurInt[i] > xmax then begin
               xmax  := voies[voie1].valeurInt[i];
               ixmax := i;
            end;
            if voies[voie1].valeurInt[i] < xmin then begin
               xmin  := voies[voie2].valeurInt[i];
               ixmin := i;
            end;
            if voies[voie2].valeurInt[i] > ymax then begin
               ymax  := voies[voie2].valeurInt[i];
               iymax := i;
            end;
            if voies[voie2].valeurInt[i] < ymin then begin
               ymin  := voies[voie2].valeurInt[i];
               iymin := i;
            end;
         end;
         if (ixmin <= ixmax) and (ixmin <= iymin) and (ixmin <= iymax)
            then i0 := ixmin
            else if (ixmax <= iymax) and (ixmax <= iymin)
               then i0 := ixmax
               else if (iymin <= iymax)
                  then i0 := iymin
                  else i0 := iymax;
      end;// ChercheIdebut

   begin  // chercheUnTour
      Fin := pred(NbrePoints);
      Debut := 0;
      if NbrePoints < 16 then exit;
      CoeffY := GetCoeff(voies[voie2].valeurInt);
      CoeffX := GetCoeff(voies[voie1].valeurInt);
      ChercheIdebut;
      i := i0;
      repeat
         Inc(i)
      until (ecart > ecartMax) or (i = NbrePoints);
 // on est loin du dpart
      if i = NbrePoints then exit;
      repeat
         Inc(i)
      until (ecart < ecartMin) or (i = NbrePoints);
 // on est prs de la boucle
      if i = NbrePoints then exit;
      Fin := i;
      MinLoc := ecartMin;
      repeat
         Inc(i);
         ecartLoc := ecart; // pour viter de calculer 3 fois
         if ecartLoc < MinLoc then begin
            minLoc := ecartLoc;
            Fin := i;
         end;// on se rapproche
      until (ecartLoc > ecartMin) or (i = NbrePoints);
// on sloigne de nouveau ou on a fini
      if Fin < NbrePoints then
         Inc(Fin); // on ferme le tour
      Debut := i0;
      if demiTour then
         Fin := Debut + (Fin - Debut) div 2;
end; // chercheUnTour

function TAgilent.GetValeur(voie : indiceVoieOsc;i : Integer): double;
begin with voies[voie] do begin
      if infiniVision then begin
         if (valeurInt[i]=0) or (valeurInt[i]=$FFFF)
             then result := Nan
             else if codage=varByte
                 then result := (valeurInt[i]-reference)*increment + origine
                 else result := (valeurInt[i]-reference)*increment + origine;
                 exit;
      end;
      if rigolDSZ then begin
         if (valeurInt[i]=0) or (valeurInt[i]=$FF)
             then result := Nan
             else result := (valeurInt[i]-reference-origine)*increment;
             exit;
      end;
      result := (reference-valeurInt[i])*increment - origine
end end;

function TAgilent.GetClipped : boolean;
var i : integer;
    voie : integer;
begin
      result := false;
      if infiniVision then begin
         for voie := voie1 to voie2 do with voies[voie] do if active then begin
         for I := debut to fin do
             if (valeurInt[i]=$0001) or (valeurInt[i]=$FFFF)
             then result := true
         end;
      end;
end;


function TAgilent.Zero(voie : indiceVoieOsc): integer;
begin with voies[voie] do
      if infiniVision
         then result := round(reference-origine/increment)
         else result := round(reference+origine/increment)
end;

function TAgilent.ValeurVolt(voie : indiceVoieOsc;valeurI : integer): double;
begin with voies[voie] do
      if infiniVision
         then if (valeurI=0) or (valeurI=$FFFF)
             then result := Nan
             else if codage=varByte
                 then result := (valeurI-reference)*increment + origine
                 else result := (valeurI-reference)*increment + origine
         else if RigolDSZ
            then result := (valeurI-origine-reference)*increment
            else result := (reference-valeurI)*increment - origine
end;

function TDataAgilent.GetValeur(voie : indiceVoieOsc;i : Integer): double;
begin with voies[voie] do
      if infiniVision
         then if (valeurInt[i]=0) or (valeurInt[i]=$FFFF)
             then result := Nan
             else if codage=varByte
                 then result := (valeurInt[i]-reference)*increment + origine
                 else result := (valeurInt[i]-reference)*increment + origine
         else if RigolDSZ
            then result := (valeurInt[i]-reference-origine)*increment
            else result := (reference-valeurInt[i])*increment - origine
end;

function TAgilent.GetValeurTemps(i : Integer): double;
begin
   with voieT do result := i*increment+origine
end;

function TAgilent.GetHole(i : Integer): boolean;
begin
   if infiniVision
      then result := (voies[voie1].valeurInt[i]=0) or
                     (voies[voie2].valeurInt[i]=0)
      else result := false
end;

function Tagilent.setChanScaleEff(Veff : double;ch : integer) : double;
var gamme : double;
    st : ansiString;
begin
    gamme := chanScaleEff(Veff);
    if gamme<0.01 then gamme := 0.01;
    if gamme>5 then gamme := 5;
    // 10 mV to 5 V when the probe attenuation factor is 1
    st := 'SCAL '+floatToStrAgilent(gamme);
    Agilent.chanCommand(st,ch);
    result := gamme;
end;

function Tagilent.setChanScaleCalibre(gamme : double;ch : integer) : double;
var st : ansiString;
begin
    gamme := chanScaleCalibre(gamme);
    if gamme<0.01 then gamme := 0.01;
    if gamme>5 then gamme := 5;
    // 10 mV to 5 V when the probe attenuation factor is 1
    st := 'SCAL '+floatToStrAgilent(gamme);
    Agilent.chanCommand(st,ch);
    result := gamme;
end;


procedure Tagilent.sauve(var data : TdataAgilent);
var v : integer;
begin
       data.debut := debut;
       data.fin := fin;
       for v:= voie1 to voie2 do begin
           data.voies[v].origine := voies[v].origine;
           data.voies[v].increment := voies[v].increment;
           data.voies[v].reference := voies[v].reference;
           data.voies[v].valeurInt := voies[v].valeurInt;
       end;
       data.NbrePoints := nbrePoints;
       data.codage := codage;
       data.infiniVision := infiniVision;
       data.rigolDSZ := RigolDSZ;
end;

procedure Tagilent.setTimeScale(freq : double);
var gamme : double;
begin
    gamme := timeScale(Freq);
    WriteString(':TIM:SCAL '+floatToStrAgilent(gamme));
    waitOPC;
end;

procedure TAgilent.waitOPC;
var len : integer;
    buf : ansiString;
begin
     buf := '0';
     repeat
         sleep(100);
         writeString('*OPC?');
         sleep(100);
         len := readString(buf);
         if (len>0) and (buf[len]=crLF)
            then buf := copy(buf,1,len-1);
         application.processMessages;
     until stop or (buf='1');
     if stop and infiniVision then io.clear;
end;

procedure TGBF.waitOPC;
const compteurMax = 10;
var len : integer;
    buf : ansiString;
    compteur : integer;
begin
     buf := '0';
     compteur := 0;
     repeat
         sleep(100);
         writeString('*OPC?');
         sleep(100);
         len := readString(buf);
         if (len>0) and (buf[len]=crLF)
            then buf := copy(buf,1,len-1);
         application.processMessages;
         inc(compteur)
     until (compteur>compteurMax) or (buf='1');
     if compteur>compteurMax then io.clear;
end;

procedure TGBF.sinus(init : boolean);
// Agilent Technologies,33220A,<serial number>,
// This program sets up a waveform by selecting the waveshape
// and adjusting the frequency, amplitude, and offset
//var z : string;
begin
(*
WriteString('FUNCtion SINusoid'); // Select waveshape
// Other options are SQUare, RAMP, PULSe, NOISe, DC, and USER
WriteString('FREQuency '+intToStr(frequency)); //Set the frequency.
WriteString('VOLTage '+floatToStrAgilent(vpp)); // Set the amplitude in Vpp.
WriteString('VOLTage:OFFSet 0'); //Set the offset in Volts
WriteString('OUTPut ON'); // Turn on the instrument output
*)
//z := 'APPL:SIN 5.0E+3, 3.0 VRMS, -2.5';
//z := 'APPL:SIN '+intToStr(frequency)+', '+floatToStrAgilent(Veff)+' VRMS';
if init then begin
   writeString('OUTP:LOAD INF');
   writeString('OUTP:SYNC ON');
   writeString('VOLT:UNIT VRMS');
end;
WriteString('APPL:SIN '+floatToStrAgilent(Ffreq)+', '+floatToStrAgilent(FVeff));
WaitOPC;
end;

procedure TAgilent.sinus(init : boolean);
begin
if InfiniVision then begin
   WriteString(':WGEN:OUTP 0'); // off
   if init then begin
      WriteString(':WGEN:OUTP:LOAD ONEM');
      WriteString(':WGEN:VOLT:OFFS 0');
   end;
   WriteString(':WGEN:FREQ '+floatToStrAgilent(Ffreq));
   WriteString(':WGEN:VOLT '+floatToStrAgilent(2*FVeff*sqrt(2))); // Vpp
   WriteString(':WGEN:FUNC SIN');
   WriteString(':WGEN:OUTP 1'); // on
end;
if RigolDSZ then begin
   WriteString(':OUTP 0'); // off
   if init then begin
      WriteString(':OUTP:IMP OMEG');
      WriteString(':VOLT:OFFS 0');
   end;
   WriteString(':FREQ '+floatToStrAgilent(Ffreq));
   WriteString(':VOLT '+floatToStrAgilent(2*FVeff*sqrt(2))); // Vpp
   WriteString(':FUNC SIN');
   WriteString(':OUTP 1'); // on
end;
end;

procedure TGBF.setVeff(avalue : double);
begin
    if avalue>VeffMax then avalue := VeffMax;
    if avalue<VeffMin then avalue := VeffMin;
    FVeff := avalue;
end;

procedure TGBF.setFrequence(avalue : double);
begin
    if avalue>1e7 then avalue := 1e7;
    if avalue<1 then avalue := 1;
    Ffreq := avalue;
end;

procedure TAgilent.setFrequence(avalue : double);
begin
    if avalue>1e7 then avalue := 1e7;
    if avalue<1 then avalue := 1;
    Ffreq := avalue;
end;

procedure TAgilent.setVeff(avalue : double);
begin
    if avalue>VeffMax then avalue := VeffMax;
    if avalue<VeffMin then avalue := VeffMin;
    FVeff := avalue;
end;

function TAgilent.isOK(z : integer) : Terreur;
begin
           case codage of
                varByte : begin
                  if z=0
                     then result := erLow
                     else if z=$FF
                        then result := erHigh
                        else result := erNo
                end;
                varWord : begin
       // 0000 = hole ; 0001 = clipped low ; FFFF = clipped high
                   if z=0
                      then result := erHole
                      else if z=1
                        then result := erLow
                        else if z=$FFFF
                           then result := erHigh
                           else result := erNo
                end;
                else result := erNo;
           end;
end;

function TDataAgilent.isOK(z : integer) : Terreur;
begin
           case codage of
                varByte : begin
                  if z=0
                     then result := erLow
                     else if z=$FF
                        then result := erHigh
                        else result := erNo
                end;
                varWord : begin
       // 0000 = hole ; 0001 = clipped low ; FFFF = clipped high
                   if z=0
                      then result := erHole
                      else if z=1
                        then result := erLow
                        else if z=$FFFF
                           then result := erHigh
                           else result := erNo
                end;
                else result := erNo;
           end;
end;

function TAgilent.getMemoryDepth(NbreVoies : integer) : integer;
var reponse : string;
begin
      if NbreVoies=3 then NbreVoies := 4;
      result := MemoryDepthUneVoie div NbreVoies;
      try
      reponse := string(getString(':ACQ:MDEP?'));
      if reponse = 'AUTO'
         then
         else begin
            result := StrToInt(reponse);
         end;
      except
      end;
end;



initialization
    agilent := TAgilent.create;
    GBF := TGBF.create;

finalization
    agilent.free;
    GBF.Free;


end.

