unit DSO1000osc;

interface

uses Windows, Forms, Classes, SysUtils,
   clipBrd, spin, Messages, registry, graphics,
   jmmUtil, hpMain, HPgrapheMult, ActnList, ImgList,
   ToolWin, Controls, StdActns, ActnMan, system.UITypes,
   Dialogs, ActnCtrls, ExtCtrls, ComCtrls, StdCtrls, math,
   AgilentU, Buttons, hpdde, System.Actions, optionsCSV, Vcl.XPStyleActnCtrls,
  System.ImageList;

type
   TDSO1000OscilloForm = class(TForm)
      ActionManager1: TActionManager;
      ImageList1: TImageList;
      RegressiAction: TAction;
      EnregistreAction: TAction;
      StopAction: TAction;
      AboutAction: TAction;
      ActionToolBar1: TActionToolBar;
      SingleAction: TAction;
    Oscillo: TPaintBox;
    StatusBarKey: TStatusBar;
    UnLockAction: TAction;
    FileSaveAction: TFileSaveAs;
    lectureRG: TRadioGroup;
    NewPageAction: TAction;
    GroupBox1: TGroupBox;
    EditComm: TEdit;
    PanelValeur: TPanel;
    Edit1: TEdit;
    GroupBox5: TGroupBox;
    NomConstEdit: TLabeledEdit;
    UniteConstEdit: TLabeledEdit;
    ValeurConstEdit: TLabeledEdit;
    XYrg: TRadioGroup;
    ProgressBar: TProgressBar;
    StatusBar: TStatusBar;
    StockageGB: TGroupBox;
    ListeCourbes: TListBox;
    Panel1: TPanel;
    AddButton: TSpeedButton;
    DeleteButton: TSpeedButton;
    LabelY: TLabel;
    LabelX: TLabel;
    Curseur: TShape;
    CommEdit: TLabeledEdit;
      procedure FormCreate(Sender: TObject);
      procedure FormActivate(Sender: TObject);
      procedure FormDestroy(Sender: TObject);
      procedure FormClose(Sender: TObject; var Action: TCloseAction);
      procedure RegressiActionExecute(Sender: TObject);
      procedure EnregistreActionExecute(Sender: TObject);
      procedure StopActionExecute(Sender: TObject);
    procedure SingleActionExecute(Sender: TObject);
    procedure OscilloPaint(Sender: TObject);
    procedure UnLockActionExecute(Sender: TObject);
    procedure FileSaveActionAccept(Sender: TObject);
    procedure NewPageActionExecute(Sender: TObject);
    procedure OscilloMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure AddButtonClick(Sender: TObject);
    procedure ListeCourbesDrawItem(Control: TWinControl; Index: Integer;
      Rect: TRect; State: TOwnerDrawState);
    procedure DeleteButtonClick(Sender: TObject);
    procedure XYrgClick(Sender: TObject);
    procedure OscilloMouseEnter(Sender: TObject);
    procedure OscilloMouseLeave(Sender: TObject);
   private
      AcqFaite, AcqAsauver: boolean;
      GrapheOsc : TgrapheOsc;
      FichierRegressiCree : boolean;
      procedure SetBoutonAcq(Permis: boolean);
      procedure EnvoieDonnees(envoi : Tenvoi);
      procedure EnvoieFichier;
   public
   protected
   end;

var
   DSO1000OscilloForm: TDSO1000OscilloForm;

implementation

{$R *.dfm}

procedure TDSO1000OscilloForm.FormCreate(Sender: TObject);
begin
   AcqFaite := False;
   AcqAsauver := False;
   agilent.progressBar := progressBar;
   GrapheOsc := TgrapheOsc.Create;
   GrapheOsc.paintBox := oscillo;
end;

procedure TDSO1000OscilloForm.SetBoutonAcq(permis: boolean);
begin
   EnregistreAction.Enabled := permis;
   RegressiAction.Enabled := permis and AcqFaite;
   FileSaveAction.Enabled := RegressiAction.Enabled;
   NewPageAction.enabled := RegressiAction.Enabled and FichierRegressiCree;
   StopAction.Enabled := not permis;
end;

procedure TDSO1000OscilloForm.SingleActionExecute(Sender: TObject);
begin
    stopAction.Enabled := true;
    UnLockAction.enabled := false;
    agilent.KeyCommand('SINGLE');
    EnregistreActionExecute(Sender);
end;


procedure TDSO1000OscilloForm.OscilloMouseEnter(Sender: TObject);
begin
   labelX.Visible := (xyrg.ItemIndex>0) and (agilent.NbrePoints>0);
   labelY.Visible := labelx.visible;
   curseur.Visible := labelx.visible;
end;

procedure TDSO1000OscilloForm.OscilloMouseLeave(Sender: TObject);
begin
   labelX.Visible := false;
   labelY.Visible := false;
   curseur.Visible := false
end;

procedure TDSO1000OscilloForm.OscilloMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
var v1,v2 : double;
begin
    if not labelx.Visible or stopAction.enabled then exit;
    grapheOsc.EcranToGrandeur(v1,v2,x,y);
    labelX.transparent := false;
    labelY.transparent := false;
    labelX.caption := 'V1='+formatCourt(v1,'V')+' ';
    labelY.caption := 'V2='+formatCourt(v2,'V')+' ';
    labelX.top := oscillo.Top+2;
    labelY.top := oscillo.Top+y-(labelY.Height div 2);
    labelY.left := 2;
    labelX.left := x-(labelX.width div 2);
    curseur.top := oscillo.Top+y-(curseur.Height div 2);
    curseur.left := x-(curseur.Width div 2)
end;

procedure TDSO1000OscilloForm.OscilloPaint(Sender: TObject);
begin
   grapheOsc.modeXY := XYrg.ItemIndex>0;
   if (XYrg.ItemIndex > 0)
      then with grapheOsc do
           agilent.chercheUnTour(XYrg.ItemIndex = 2);
   GrapheOsc.DrawG;
end;

procedure TDSO1000OscilloForm.FormActivate(Sender: TObject);
begin
   setBoutonAcq(True);
   agilent.writeString(':BEEP:ENAB OFF');
end;

procedure TDSO1000OscilloForm.FormDestroy(Sender: TObject);
begin
   DSO1000OscilloForm := nil;
end;

procedure TDSO1000OscilloForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
   if AcqAsauver and
      (MessageDlg('Sauvegarde', mtConfirmation, [mbYes, mbNo], 0) = idYes)
   then FileSaveAction.Execute;
   AgilentFormMain.setMode(mNone);
end;

procedure TDSO1000OscilloForm.ListeCourbesDrawItem(Control: TWinControl;
  Index: Integer; Rect: TRect; State: TOwnerDrawState);
var Offset : integer;
begin with (Control as TListBox).Canvas do begin
// draw on control canvas, not on the form
    brush.Color := clBlack;
    brush.Style := bsSolid;
    FillRect(Rect);
    Offset := 2; // provide default offset
    font.Color := couleurPale(couleurXY,index);
    TextOut(Rect.Left + Offset, Rect.Top, (Control as TListBox).Items[Index])
end end;

procedure TDSO1000OscilloForm.NewPageActionExecute(Sender: TObject);
begin
    envoieDonnees(eAdd)
end;

procedure TDSO1000OscilloForm.RegressiActionExecute(Sender: TObject);
begin
    if stockageGB.Visible and (grapheOsc.oldCourbes.Count>0)
       then envoieFichier
       else envoieDonnees(eLoad)
end;

procedure TDSO1000OscilloForm.EnvoieFichier;
var
    fichier : textFile;
    nomFichier : String;

procedure writeNombreWin(const z : double);
var nombre : string;
    posSeparator : integer;
begin
     nombre := FloatToStrF(z,ffGeneral,7,2);
     if formatSettings.decimalSeparator<>'.' then begin
        posSeparator := pos(formatSettings.decimalSeparator,nombre);
        if posSeparator>0 then Nombre[posSeparator] := '.';
     end;
     write(fichier,nombre,' ')
end;

Procedure EcritGraphe;
begin with GrapheOsc,Agilent do begin
   writeln(fichier,symbReg,'0 GRAPHE VAR');
   writeln(fichier,symbReg2,1,' TRACE');
   writeln(fichier,'2');
   writeln(fichier,symbReg2,1,' COULEUR');
   writeln(fichier,couleurXY);
   writeln(fichier,symbReg2,1,' X');
   writeln(fichier,voies[voie1].nom);
   writeln(fichier,symbReg2,1,' Y');
   writeln(fichier,voies[voie2].nom);
end end;  // ecritGraphe

Procedure EcritBase;
var j,v : integer;
begin with grapheOsc,Agilent do begin
    writeln(fichier,symbReg,'1 PAGE COMMENT');
    writeln(fichier,'Oscillo XY');
    writeln(fichier,symbReg2,IntToStr(fin-debut+1)+' VALEUR VAR');
    for j := debut to fin do begin
        write(fichier,j-debut,' ');
        for v := voie1 to voie2 do
           writeNombreWin(valeurs[v,j]);
        writeln(fichier);
    end; // for j
end end;

Procedure EcritPage(page : integer);
var j,v : integer;
begin with grapheOsc,TDataAgilent(oldCourbes[page]) do begin
    writeln(fichier,symbReg,'1 PAGE COMMENT');
    writeln(fichier);
    writeln(fichier,symbReg2,IntToStr(fin-debut+1)+' VALEUR VAR');
    for j := debut to fin do begin
       write(fichier,j-debut,' ');
       for v := voie1 to voie2 do
           writeNombreWin(valeurs[v,j]);
       writeln(fichier);
    end; // for j
end end; // ecritPage

Procedure ecritEnTete;
var i,v : integer;
begin // ecritEnTete
with grapheOsc,Agilent do begin
   writeln(fichier,'EVARISTE REGRESSI WINDOWS 1.0');
   writeln(fichier,symbReg,3,' NOM VAR');
   writeln(fichier,'num');
   for v := voie1 to voie2 do with voies[v] do
       writeln(fichier,'V'+intToStr(v));
   writeln(fichier,symbReg,3,' GENRE VAR');
   for i := 1 to 3 do
       writeln(fichier,0);
   writeln(fichier,symbReg,3,' UNITE VAR');
   writeln(fichier,'');
   for v := voie1 to voie2 do with voies[v] do
       writeln(fichier,'V');
   writeln(fichier,symbReg,3,' PRECISION VAR');
   for i := 1 to 3 do
       writeln(fichier,4);
   writeln(fichier,symbReg,3,' SIGNIF VAR');
   writeln(fichier,'index d''acquisition');
   for v := voie1 to voie2 do with voies[v] do
       writeln(fichier,'tension');
   writeln(fichier,symbReg,'2 ACQUISITION');
   writeln(fichier,'Externe');
   writeln(fichier,application.exeName);
end end; // ecritEnTete

var i : integer;
begin // EnvoieFichier
     Screen.Cursor := crHourGlass;
     NomFichier := AppDataDir+'DSO.rw3';
     AssignFile(fichier,NomFichier);
     Rewrite(fichier);
     ecritEnTete;
     ecritGraphe;
     ecritBase;
     for i := 0 to pred(grapheOsc.oldCourbes.Count) do ecritPage(i);
     CloseFile(fichier);
     FormDDE.Execute(eFileLoad,nil,nomFichier);
     Screen.Cursor := crDefault;
end; // EnvoieFichier

procedure TDSO1000OscilloForm.EnvoieDonnees(envoi : Tenvoi);
var
   donnees: TStringList;

   procedure AjouteConstOsc;
   var
      valeurConst: double;
   begin
         if NomConstEdit.Text = '' then exit;
         ValeurConst := StrToFloatWin(ValeurConstEdit.Text);
         if isNan(valeurConst) then exit;
         donnees.Add('');
         donnees.Add(UnicodeVersRW3(NomConstEdit.Text));
         donnees.Add(UnicodeVersRW3(UniteConstEdit.Text));
//       if CommEdit.Text<>'' then donnees.Add(UnicodeVersRW3(CommEdit.Text));
         donnees.Add(valeurConstEdit.Text);
   end;

var
   i: indiceVecteurOsc;
   v: integer;
   ligne: String;
   N: integer;
begin
   Screen.Cursor := crHourGlass;
   if (XYrg.ItemIndex > 0) then
      agilent.chercheUnTour(XYrg.ItemIndex = 3);
   N := 0;
   for v := voie1 to voie2 do
       if agilent.voies[v].active then inc(N);
   donnees := TStringList.Create;
   donnees.Add(Application.exeName);
   if envoi = eLoad
      then donnees.Add('Acquisition par ' + Agilent.identificateur)
      else donnees.Add('');
   donnees.Add(unicodeVersRW3(EditComm.Text));
   ligne := ligne + unicodeVersRW3(agilent.voieT.nom);
   for v := voie1 to voie2 do
      with agilent.voies[v] do
         if active then
            ligne := ligne +crTab+ unicodeVersRW3(nom);
   donnees.Add(ligne);
   Ligne := unicodeVersRW3(agilent.voieT.nomUnite);
   for v := voie1 to voie2 do
      with agilent.voies[v] do
         if active then
            ligne := ligne + crTab+unicodeVersRW3(nomUnite);
   donnees.Add(ligne);
   Ligne := unicodeVersRW3(agilent.voieT.signif);
   for v := voie1 to voie2 do
      with agilent.voies[v] do
         if active then
            ligne := ligne + crTab+unicodeVersRW3(signif);
   donnees.Add(ligne);
   for i := agilent.Debut to agilent.Fin do begin
      ligne := floatToStrPoint(agilent.ValeursT[i]);
      for v := voie1 to voie2 do
          if agilent.voies[v].active then
               ligne := ligne + crTab + floatToStrPoint(agilent.Valeurs[v,i]);
      donnees.Add(ligne);
   end;
   AjouteConstOsc;
   if (XYrg.ItemIndex > 0) and
      (envoi = eLoad) then begin
      donnees.Add(symbReg2 + '1 NOMX');
      donnees.Add(unicodeVersRW3(agilent.voies[voie1].nom));
      donnees.Add(symbReg2 + '1 NOMY');
      donnees.Add(unicodeVersRW3(agilent.voies[voie2].nom));
   end;
    if (envoi = eLoad) then begin
      donnees.Add(symbReg2+intToStr(N)+' COULEUR');
      for i := 1 to N do donnees.Add(IntToStr(couleurAgilent[i]));
      donnees.Add(symbReg2+intToStr(N)+' LIGNE');
      for i := 1 to N do donnees.Add('2');
   end;
   FormDDE.Execute(envoi, donnees,'');
   FichierRegressiCree := true;
   Donnees.Free;
   Screen.Cursor := crDefault;
   AcqAsauver := False;
end;

procedure TDSO1000OscilloForm.FileSaveActionAccept(Sender: TObject);
var
   fichier: textFile;
   nomFichier : string;

Procedure sautDeLigne;
begin
    write(Fichier,crLF)
end;

   procedure EcritPage;
   var
      j: indiceVecteurOsc;
      v: indiceVoieOsc;
   begin
         for j := agilent.debut to agilent.fin do begin
            Write(fichier, FloatToStr(agilent.valeursT[j]));
            for v := voie1 to voie2 do if agilent.voies[v].active then
                Write(fichier, separeValue,FloatToStr(agilent.valeurs[v,j]));
            sautDeLigne;
         end;
   end; // ecritPage

   procedure ecritEnTete;
   var
      j: integer;
   begin
         Write(fichier, agilent.voieT.nom);
         for j := voie1 to Voie2 do
            if agilent.voies[j].active then
               Write(fichier, separeValue, agilent.voies[j].nom);
         sautDeLigne;
         Write(fichier, agilent.voieT.nomUnite);
         for j := voie1 to Voie2 do
            if agilent.voies[j].active then
               Write(fichier, separeValue, agilent.voies[j].nomUnite);
         sautDeLigne;
   end; // ecritEnTete

var extension : string;
begin
   Screen.Cursor := crHourGlass;
   nomFichier := FileSaveAction.Dialog.FileName;
   extension := AnsiUpperCase(ExtractFileExt(NomFichier));
   if extension='' then extension := '.CSV';
   nomFichier := nomFichier+extension;
   if (XYrg.ItemIndex > 0)
      then agilent.chercheUnTour(XYrg.ItemIndex = 2);
   AssignFile(fichier, NomFichier);
   Rewrite(fichier);
   if not OptionsCSVForm.DataOnly then ecritEnTete;
   ecritPage;
   CloseFile(fichier);
   Screen.Cursor := crDefault;
   AcqAsauver := False;
end; // SaveAction

procedure TDSO1000OscilloForm.AddButtonClick(Sender: TObject);
var newdata : TdataAgilent;
begin
     newData := TdataAgilent.create;
     Agilent.sauve(newData);
     grapheOsc.oldCourbes.add(newData);
     listeCourbes.items.Add('Courbe');
     DeleteButton.Visible := true;
     AddButton.Visible := listeCourbes.count<5;
end;

procedure TDSO1000OscilloForm.DeleteButtonClick(Sender: TObject);
var data : TdataAgilent;
begin
    if listeCourbes.ItemIndex<0 then listeCourbes.ItemIndex := 0;
    data := TdataAgilent(grapheOsc.oldCourbes[listeCourbes.ItemIndex]);
    data.Free;
    grapheOsc.oldCourbes.delete(listeCourbes.itemIndex);
    listeCourbes.DeleteSelected;
    DeleteButton.Visible := listeCourbes.count>0;
    AddButton.Visible := listeCourbes.count<5;
    oscillo.Repaint;
end;

procedure TDSO1000OscilloForm.EnregistreActionExecute(Sender: TObject);
var reponseTrig,reponseAcq : string;
    RunTodo,isPeak : boolean;
begin
   unlockAction.Enabled := true;
   setBoutonAcq(False);
   RunTodo := false;
   statusBar.simpleText := '';
   reponseTrig := string(agilent.getString(':TRIG:STAT?')); // OK Rigol
//   if reponseTrig='RUN' then ; // RUN  The oscilloscope is running.
//   if reponseTrig='STOP' then ;; // STOP  The oscilloscope is stopped.
   if reponseTrig='T''D' then statusBar.simpleText := 'Remplissage de la mmoire de l''oscillo en cours';;
// T'D  The oscilloscope has found the trigger condition and is filling the rest of acquisition memory.
   if reponseTrig='WAIT' then statusBar.simpleText := 'Attente du dclenchement de l''oscillo';;
// WAIT  In the NORMal trigger sweep mode the oscilloscope is waiting to find the trigger condition.
//   if reponseTrig='SCAN' then statusBar.simpleText := '';; // SCAN
   if reponseTrig='AUTO' then statusBar.simpleText := 'Remplissage de la mmoire de l''oscillo en cours';
// AUTO - a trigger has been forced and the oscilloscope is filling the rest of acquisition memory.
   if (reponseTrig<>'STOP') then begin
        agilent.writeString(':STOP'); // OK Rigol
        agilent.waitOPC;
        RunTodo := true;
   end;
   statusBar.simpleText := 'Lecture de la mmoire de l''oscillo en cours';
   reponseAcq := string(agilent.getString(':ACQ:TYPE?'));
   isPeak := pos('PEAK',reponseAcq)>0;
   if isPeak
     then begin
        agilent.litValeurs(wMaxPeak);
        agilent.litValeurs(wMinPeak);
     end
     else case lectureRG.ItemIndex of
         0 : agilent.litValeurs(wScreen);
         1 : agilent.litValeurs(wMemory);
     end;
   if agilent.pbDataByte then showMessage('Saturation ?');
   AcqFaite := agilent.nbrePoints>16;
   stopAction.enabled := false;
   if RunTodo then begin
        agilent.writeString(':RUN');  // OK Rigol
        agilent.waitOPC;
        statusBar.simpleText := 'Fin de lecture de la mmoire de l''oscillo';
   end;
   agilent.unLock;
   oscillo.refresh;
   setBoutonAcq(True);
end;

procedure TDSO1000OscilloForm.StopActionExecute(Sender: TObject);
begin
   agilent.stop := True
end;

procedure TDSO1000OscilloForm.UnLockActionExecute(Sender: TObject);
begin
   agilent.unlock;
   unlockAction.Enabled := false;
end;

procedure TDSO1000OscilloForm.XYrgClick(Sender: TObject);
begin
   stockageGB.Visible := xyrg.ItemIndex>0
end;

end.

