unit Hpmain;

interface

uses Windows, Classes, Graphics, Forms, Controls, Menus,
  Dialogs, StdCtrls, Buttons, ExtCtrls, sysUtils, Messages,
  clipBrd, spin, registry, ComCtrls,
  oneinst, jmmUtil, optionscsv;

type
  Tmode = (mNone,mOsc,mMult,mBode,mPhase);

  TAgilentFormMain = class(TForm)
    BoutonPanel: TPanel;
    OscBtn: TBitBtn;
    ComBtn: TBitBtn;
    HelpBtn: TBitBtn;
    AboutBtn: TBitBtn;
    MultBtn: TBitBtn;
    Error1000: TStatusBar;
    ExitBtn: TBitBtn;
    GBFcb: TComboBox;
    Oscillocb: TComboBox;
    GBFTitre: TLabel;
    OscilloTitre: TLabel;
    BodeBtn: TBitBtn;
    csvBtn: TBitBtn;
    procedure ExitItemClick(Sender: TObject);
    procedure AboutItemClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure OscBtnClick(Sender: TObject);
    procedure MultBtnClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure ComBtnClick(Sender: TObject);
    procedure HelpBtnClick(Sender: TObject);
    procedure BodeBtnClick(Sender: TObject);
    procedure csvBtnClick(Sender: TObject);
  private
    grecActif : boolean;
    procedure verifVisa;
  public
    AcquisitionEnCours : Tmode;
    procedure AppMessage(var Msg: TMsg; var Handled: Boolean);
    procedure setFocusMain;
    Procedure SetMode(Acq : Tmode);
  protected
    procedure WMFocus(var Msg: Tmessage); message wm_Focus;
  end;

var
  AgilentFormMain: TAgilentFormMain;

implementation

uses agilentU, HPabout, hpmult, hposc, hpdde, DSO1000osc, libusb, HelpFormU,
  DSOX2000, HPbode;

{$R *.DFM}

procedure TAgilentFormMain.AppMessage(var Msg: TMsg; var Handled: Boolean);

const
        GrecMaj : array[1..26] of char =
  (AlphaMaj,BetaMaj,ChiMaj,deltaMaj,EpsilonMaj,phiMaj,gammaMaj,etaMaj,
   iotaMaj,psiMaj,kappaMaj,lambdaMaj,MuMaj,NuMaj,omicronMaj,piMaj,thetaMaj,
   rhoMaj,sigmaMaj,tauMaj,upsilonMaj,videMaj,omegaMaj,xiMaj,'Y',zetaMaj);

        GrecMin : array[1..26] of char =
  (alpha,beta,chi,delta,epsilon,phi,gamma,eta,iota,psi,kappa,lambda,
   mu,nu,omicron,piMin,theta,rho,sigmaMin,tau,upsilon,vide,omega,xi,'y',zeta);

var codeCarac : integer;
begin with Msg do if Message=wm_char then
      if (Wparam=7) and (GetKeyState(vk_control)<0)  // Ctrl+G
            then GrecActif := true
            else begin
                if GrecActif then begin
                   CodeCarac := Wparam-ord('A')+1;
                   if (CodeCarac<=26) and (codeCarac>0) then Wparam := ord(GrecMaj[CodeCarac])
                      else begin
                        CodeCarac := Wparam-ord('a')+1;
                        if (CodeCarac<=26) and (codeCarac>0) then Wparam := ord(GrecMin[codeCarac]);
                      end;
                   GrecActif := false;
                end;
           end
end;

procedure TAgilentFormMain.BodeBtnClick(Sender: TObject);
begin
    if bodeForm=nil then
       Application.createForm(TbodeForm,bodeForm);
    setMode(mBode);
    BodeForm.GBFinterne := agilent.open and not gbf.open and
       (agilent.infiniVision or agilent.rigolDSZ);
    BodeForm.show;
    BodeForm.SetFocus;
end;

procedure TAgilentFormMain.ComBtnClick(Sender: TObject);
begin
    verifVisa;
    if ListeVisaCom.count=1 then exit;// automatique
    if listeVisaCom.Count>1
       then agilent.init(ListeVisaCom[oscilloCB.itemIndex].nom)
       else agilent.init(''); // adresseUSB.text
    if agilent.open
       then Caption := 'Acquisition avec '+string(agilent.identificateur)
       else begin
           Caption := 'Acquisition avec oscilloscope Agilent/Keysight';
           erreur('Impossible de se connecter  l''oscilloscope');
       end;
    if gbfCB.Visible then GBF.init(ListeVisaCom[gbfcb.itemIndex].Nom);
    setMode(mNone);
end;

procedure TAgilentFormMain.csvBtnClick(Sender: TObject);
begin
    if optionsCsvForm=nil
       then Application.createForm(ToptionsCsvForm,optionsCsvForm);
    optionsCsvForm.showModal
end;

procedure TAgilentFormMain.ExitItemClick(Sender: TObject);
begin
   Close
end;

procedure TAgilentFormMain.AboutItemClick(Sender: TObject);
begin
    if aboutBox=nil
        then Application.createForm(TAboutBox,aboutbox);
    AboutBox.ShowModal
end;

procedure TAgilentFormMain.FormCreate(Sender: TObject);
begin
  AcquisitionEnCours := mNone;
  Application.ShowHint := true;
  Application.OnMessage := AppMessage;
end;

procedure TAgilentFormMain.HelpBtnClick(Sender: TObject);
begin
 if helpForm=nil
        then Application.createForm(THelpForm,helpForm);
  HelpForm.showModal
end;

procedure TAgilentFormMain.SetMode(Acq : Tmode);
begin
     if agilent.open
        then AcquisitionEnCours := Acq
        else AcquisitionEnCours := mNone;
     Error1000.Visible := ErrorVisa<>'';
     Error1000.simpleText := ErrorVisa;
     // Add Erreur1000
     OscBtn.enabled := agilent.open and (Acq in [mNone,mOsc]);
     MultBtn.enabled := agilent.open and (Acq in [mNone,mMult]);
     BodeBtn.enabled := agilent.open and
              (gbf.open or agilent.infiniVision or  agilent.rigolDSZ) and
              (Acq in [mNone,mBode]);
end;

procedure TAgilentFormMain.FormActivate(Sender: TObject);
begin
        verifVisa;
end;

procedure TAgilentFormMain.VerifVisa;
var i : integer;
begin
     setNomVisa;
     Error1000.Visible := ErrorVisa<>'';
     Error1000.simpleText := ErrorVisa;
     BodeBtn.enabled := agilent.infiniVision or agilent.rigolDSZ or (ListeVisaCom.count>1);
     GBFcb.Visible := ListeVisaCom.count>1;
     GBFTitre.Visible := ListeVisaCom.count>1;
     OscilloCB.Visible := ListeVisaCom.count>1;
     OscilloTitre.Visible := ListeVisaCom.count>1;
     GBFCB.Items.Clear;
     OscilloCB.items.clear;
     for I := 0 to listeVisaCom.count-1 do begin
          GBFcb.items.Add(ListeVisaCom[i].Nom);
          Oscillocb.items.Add(ListeVisaCom[i].Nom);
     end;
     if ListeVisaCom.count>1 then begin
        if (pos('MY',ListeVisaCom[0].Nom)>0) or
           (pos('my',ListeVisaCom[0].Nom)>0)
            then GBFcb.ItemIndex := 0
            else GBFcb.ItemIndex := 1;
        if (pos('CN',ListeVisaCom[0].nom)>0) or
           (pos('cn',ListeVisaCom[0].Nom)>0)
           then Oscillocb.ItemIndex := 0
           else Oscillocb.ItemIndex := 1;
     end
     else begin
         Oscillocb.ItemIndex := 0;
         GBFcb.ItemIndex := 0;
     end;
     if (ListeVisaCom.count=1) then begin
         agilent.init(ListeVisaCom[0].nom);
         if agilent.open then begin
             if agilent.identificateur=''
             then Caption := 'Pas de rponse  *IDN?'
             else Caption := 'Acquisition avec '+string(agilent.identificateur);
         end;
     end;
     setMode(mNone);
end;


procedure TAgilentFormMain.OscBtnClick(Sender: TObject);
begin
    if ((agilent.usb=oLibUSB) or (agilent.usb=oRigol)) and (oscilloForm=nil)
        then Application.createForm(ToscilloForm,oscilloForm); // DSO3000 et Rigol
    if (agilent.usb=oVisaCOM) then if agilent.InfiniVision or agilent.RigolDSZ
        then if (DSOX2000oscilloForm=nil)
           then Application.createForm(TDSOX2000oscilloForm,DSOX2000oscilloForm)  // InfiniVision
           else
        else if (DSO1000oscilloForm=nil)
           then Application.createForm(TDSO1000oscilloForm,DSO1000oscilloForm);
    setMode(mOsc);
    if (agilent.usb=oLibUSB) or (agilent.usb=oRigol) then begin
       OscilloForm.show;
       OscilloForm.setFocus;
    end;
    if agilent.usb=oVisaCOM then if agilent.infiniVision or agilent.RigolDSZ
       then begin
          DSOX2000OscilloForm.show;
          DSOX2000OscilloForm.setFocus;
       end
       else begin
          DSO1000OscilloForm.show;
          DSO1000OscilloForm.setFocus;
       end;
end;

procedure TAgilentFormMain.MultBtnClick(Sender: TObject);
begin
     if multimetreForm=nil
        then Application.createForm(TMultimetreForm,multimetreForm);
     setMode(mMult);
     MultimetreForm.show;
     MultimetreForm.setFocus;
end;                                                       

procedure TAgilentFormMain.setFocusMain;
label fin;
begin
     try
     case AcquisitionEnCours of
        mMult : begin
            if multimetreForm=nil then begin
                   setMode(mNone);
                   goto fin;
               end;
            if not multimetreForm.showing then multimetreForm.show;
            if not multimetreForm.active then multimetreForm.setFocus;
        end;
        mBode : begin
            if bodeForm=nil then begin
                   setMode(mNone);
                   goto fin;
               end;
            if not bodeForm.showing then bodeForm.show;
            if not bodeForm.active then bodeForm.setFocus;
        end;
        mOsc : begin
        if (agilent.usb=oLibUSB) or (agilent.usb=oRigol) then begin
            if oscilloForm=nil
               then begin
                   setMode(mNone);
                   goto fin;
               end;
            if not oscilloForm.showing then oscilloForm.show;
            if not oscilloForm.active then oscilloForm.setFocus;
        end;
        if (agilent.usb=oVisaCOM) then if agilent.infiniVision or agilent.RigolDSZ
        then begin
            if DSOX2000oscilloForm=nil
               then begin
                   setMode(mNone);
                   goto fin;
               end;
            if not DSOX2000oscilloForm.showing then DSOX2000oscilloForm.show;
            if not DSOX2000oscilloForm.active then DSOX2000oscilloForm.setFocus;
        end
        else begin
             if DSO1000oscilloForm=nil
               then begin
                   setMode(mNone);
                   goto fin;
               end;
            if not DSO1000oscilloForm.showing then DSO1000oscilloForm.show;
            if not DSO1000oscilloForm.active then DSO1000oscilloForm.setFocus;
        end;
        end;
     end;
     fin :
     if acquisitionEnCours=mNone then setFocus;
     except
     end;
end;

procedure TAgilentFormMain.WMFocus(var Msg: Tmessage);
begin
   setFocusMain;
   Msg.result := 0;
end;

end.


