unit HPgrapheMult;

interface

uses windows, SysUtils, Messages, Graphics, Classes, math,
   Forms, Controls, Buttons, StdCtrls, ExtCtrls, Dialogs,
   jmmUtil, agilentU, fft, optionsCSV, statCalc;

const
   WM_Acq_Point = WM_USER + 4;
   WM_Acq_Raz = WM_USER + 5;
   MaxVecteur  = 256;
   MaxVoieMeasure = 14;
   MaxCourbes = 4;
   CouleurAxe: Tcolor = clSilver;
   CouleurXY: Tcolor = clRed;
   couleurAgilent : array[1..2] of Tcolor = (clYellow,clLime);
   VoiePhase = 3;
   VoieGain = 4;
   Voie1 = 1;
   Voie2 = 2;
   voie3 = 3;
   voie4 = 4;
   voieF = 0;

type
   TCalculIncertitude = (ciNone, ciFreq, ciTemps, ciTension, ciDuty,
      ciPhase, ciTensionEff, ciGain);
   indiceVecteur  = 0..MaxVecteur;
   vecteur        = array[indiceVecteur] of double;
   indiceVoieMult = voieF..VoieGain;

   TuniteDeBase = (_metre,_kilogramme,_seconde,_ampere,_kelvin,_mole,_candela);

   TvoieRecord = record
      mini, maxi: double;
      Nom: string;
      Command : ansiString;
      nomUnite, Signif: string;
      Positive:  boolean;
      CalculIncertitude: TCalculIncertitude;
      NumVoie: integer;
   end;

   TvoieAgilent = record
      mini, maxi: double;
      Nom: string;
      Command : ansiString;
      nomUnite, Signif: string;
      Positive : boolean;
      CalculIncertitude: TCalculIncertitude;
   end;

   Tvoie = class
      a, b: double; { entier sur cran = rel*a+b }
      valeurCourante, oldValeur: double;
      calibre : double;
      incertitudeCourante: double;
      fullScale,offset: double;
      incertitudeType: double;
      valeur:      vecteur;
      incertitude: vecteur; // mode multimtre
      AxeLog :   boolean;
      LabelVoie:   Tlabel;
      GBvoie:      TgroupBox;
      posCurseur:  integer;
      mini, maxi, miniLog: double;
      Nom: string;
      Command: AnsiString;
      nomUnite, Signif, commentaire : string;
      Positive:  boolean;
      CalculIncertitude: TCalculIncertitude;
      erreurLecture:     boolean;
      NumVoie:  integer;
      active : boolean;
      color : Tcolor;
      stat : TcalculStatistique;
      constructor Create;
      destructor Destroy; override;
      function GrandeurToEcran(x: double): integer;
      procedure SetIncertitude;
      procedure SetIncertitudeDSOX;

      function StrValeur(nombre: double): string;
      function AffVoltmetre: string;
   end;

   TgrapheMult = class
      dimY,dimX:  integer;
      paintBox:    TpaintBox;
      Quadrillage:    boolean;
      ErreurLectureGlb: boolean;
      voies:  array[indiceVoieMult] of Tvoie;
      NbrePoints:   integer;
      LimiteCourbe:   Trect;
      avecAxeY:   boolean;
      constructor Create;
      destructor Destroy;override;
      procedure DrawG(avecCurseur : boolean);
      procedure EnregistrePointCourant;
      procedure TraceCroix(xr, yr: double; v: indiceVoieMult);
      procedure DrawAxis;
      procedure SaveData(const NomFichier: string);
      procedure LineG(x1, y1, x2, y2: integer);
      procedure resetStat;
      procedure tri;
   end;

   TgrapheOsc = class
      PaintBox: TPaintBox;
      Ax,Ay,Bx,By : double;
      modeXY : boolean;
      oldCourbes : Tlist;
      constructor Create;
      destructor Destroy;override;
      procedure DrawG;
      procedure GrandeurToEcran(x,y: double;var xi,yi : integer);
      procedure EcranToGrandeur(var vx,vy: double;var xi,yi : integer);
   end;

   function CouleurPale(color1: Tcolor;k : integer): Tcolor;

const
   ChiffreEtc = ['0'..'9', '+', '-', 'e', 'E', '.',','];
   VoieDefaut: array[indiceVoieMult] of TvoieRecord =
      ((mini: 100; maxi: 10000; Nom: 'f'; command: 'FREQ';
      nomUnite: 'Hz'; Signif: 'frquence'; positive: false;
      calculIncertitude: ciFreq; Numvoie: 1),
      (mini: 0; maxi: 5; Nom: 'Ve'; command: 'VRMS';
      nomUnite: 'V'; signif: 'tension efficace'; positive: True;
      calculIncertitude: ciTensionEff; Numvoie: 1),
      (mini: 0; maxi: 5; Nom: 'Vs'; command: 'VRMS';
      nomUnite: 'V'; signif: 'tension efficace'; positive: True;
      calculIncertitude: ciTensionEff; Numvoie: 2),
      (mini: -185; maxi: +185; Nom: phi; command: 'PPHAS';
      nomUnite: ''; signif: 'phase (1,2)'; positive: false;
      calculIncertitude: ciPhase; Numvoie: 1),
      (mini: 0; maxi: +10; Nom: 'G'; command: '';
      nomUnite: ''; signif: 'Gain'; positive: true;
      calculIncertitude: ciGain; Numvoie: 1));

   VoiesAgilent: array[0..MaxVoieMeasure] of TvoieAgilent =
      ((mini: -5; maxi: 5; Nom: 'Vav'; command: 'VAV';
      nomUnite: 'V'; Signif: 'tension moyenne'; positive: True;
      calculIncertitude: ciTension),
      (mini: 0; maxi: 5; Nom: 'Veff'; command: 'VRMS';
      nomUnite: 'V'; signif: 'tension efficace'; positive: True;
      calculIncertitude: ciTensionEff),
      (mini: 0; maxi: 5; Nom: 'Vamp'; command: 'VAMP';
      nomUnite: 'V'; signif: 'double de l''amplitude'; positive: True;
      calculIncertitude: ciTensionEff),
      (mini: 0; maxi: 5; Nom: 'Vpp'; command: 'VPP';
      nomUnite: 'V'; signif: 'tension crte  crte'; positive: True;
      calculIncertitude: ciTensionEff),
      (mini: -5; maxi: 5; Nom: 'Vbas'; command: 'VBAS';
      nomUnite: 'V'; signif: 'tension basse'; positive: False;
      calculIncertitude: ciTension),
      (mini: -5; maxi: 5; Nom: 'Vtop'; command: 'VTOP';
      nomUnite: 'V'; signif: 'tension haute'; positive: False;
      calculIncertitude: ciTension),
      (mini: -5; maxi: 5; Nom: 'Vmax'; command: 'VMAX';
      nomUnite: 'V'; signif: 'tension maxi'; positive: False;
      calculIncertitude: ciTension),
      (mini: -5; maxi: 5; Nom: 'Vmin'; command: 'VMIN';
      nomUnite: 'V'; signif: 'tension mini'; positive: False;
      calculIncertitude: ciTension),
      (mini: 0; maxi: 1; Nom: 'ln'; command: 'NWID';
      nomUnite: 's'; signif: 'largeur ngative'; positive: True;
      calculIncertitude: ciTemps),
      (mini: 0; maxi: 1; Nom: 'lp'; command: 'PWID';
      nomUnite: 's'; signif: 'largeur positive'; positive: True;
      calculIncertitude: ciTemps),
      (mini: 0; maxi: 1; Nom: 'td'; command: 'FALL';
      nomUnite: 's'; signif: 'temps de descente'; positive: True;
      calculIncertitude: ciTemps),
      (mini: 0; maxi: 1; Nom: 'tm'; command: 'RSI';
      nomUnite: 's'; signif: 'temps de monte'; positive: True;
      calculIncertitude: ciTemps),
      (mini: 0; maxi: 100; Nom: 'over'; command: 'OVER';
      nomUnite: '%'; signif: 'dpassement'; positive: True;
      calculIncertitude: ciDuty),
      (mini: 0; maxi: 100; Nom: 'alpha'; command: 'NDUT';
      nomUnite: '%'; signif: 'rapport cyclique (ngatif)'; positive: True;
      calculIncertitude: ciDuty),
      (mini: 0; maxi: 100; Nom: 'alpha'; command: 'PDUT';
      nomUnite: '%'; signif: 'rapport cyclique'; positive: true;
      calculIncertitude: ciDuty));

var
   BaseDeTemps:       double;
   FrequenceCourante: double;

implementation

function CouleurPale(color1: Tcolor;k : integer): Tcolor;
var
  R, G, B, decale : byte;
begin
      decale := (k+1)*48;
      B := getBvalue(color1);
      if B > decale then B := B - decale else B := 0;
      R := getRvalue(color1);
      if R > decale then R := R - decale else R := 0;
      G := getGvalue(color1);
      if G > decale then G := G - decale else G := 0;
      Result := RGB(R, G, B);
end;

function FormatCourt(Nombre: double; NomUnite: string): string;
var
   unite:  coeffUnite;
   Negatif: boolean;
   tampon: string;
begin
   if isNan(nombre) then
      FormatCourt := ' '
   else begin
      negatif := nombre < 0;
      nombre := abs(nombre);
      unite := nulle;
      while (nombre >= 999) and (unite < infini) do begin
         nombre := nombre / 1000;
         Inc(unite);
      end; { nombre<1000 ou infini }
      while (nombre < 0.999) and (unite > zero) do begin
         nombre := nombre * 1000;
         Dec(unite);
      end; { nombre>=1 ou zero }
      if unite in [infini, zero] then
         tampon := '0 '
      else begin
         tampon := FloatToStrF(nombre, ffGeneral, 4, 0);
         if negatif then
            tampon := '-' + tampon;
         tampon := tampon + ' ';
         if unite <> nulle then
            tampon := tampon + symboleUnite[unite];
      end;
      FormatCourt := tampon + NomUnite;
   end;
end;

constructor Tvoie.Create;
var i : integer;
begin
   inherited Create;
   A := 1;
   B := 0;
   ValeurCourante := 1;
   oldValeur := Nan;
   incertitudeCourante := 0;
   calculIncertitude := ciNone;
   NumVoie := 1;
   AxeLog  := False;
   active := true;
   nom  := '';
   Signif := '';
   nomUnite := '';
   Command := '';
   Maxi := 1;
   Mini := 0;
   MiniLog := 0.01;
   Positive := True;
   CalculIncertitude := ciNone;
   BaseDeTemps := 0.01;
   for i := 0 to maxVecteur do
       valeur[i] := 0;
   stat := TcalculStatistique.Create;
end; //  Tvoie.Create

destructor Tvoie.Destroy;
begin
   stat.Free
end;

function Tvoie.GrandeurToEcran(x: double): integer;
begin
   if isNan(x)
   then Result := -1
   else begin
      if axeLog then x := log10(x);
      GrandeurToEcran := round(A * x + B);
   end;
end;

procedure TgrapheMult.resetStat;
var i : indiceVoieMult;
begin
 for i := voieF to VoieGain do
       voies[i].Stat.reset;
end;

procedure TgrapheOsc.GrandeurToEcran(x,y: double;var xi,yi : integer);
begin
   if isNan(x) or isNan(y) then begin
      xi := 0;
      yi := 0;
   end
   else begin
      xi := round(Ax * x + Bx);
      yi := round(Ay * y + By);
   end;
end;

procedure TgrapheOsc.EcranToGrandeur(var vx,vy: double;var xi,yi : integer);
var i,i1 : integer;
    x1,y1 : integer;
    mini : integer;
    x,y : integer;
begin with agilent do begin
   mini := 256;
   i1 := -1;
   for i := debut to fin do begin
       grandeurToEcran(voies[voie1].valeurInt[i],
                       voies[voie2].valeurInt[i],x1,y1);
       if (abs(xi-x1)+abs(yi-y1))<mini then begin
          mini := abs(xi-x1)+abs(yi-y1);
          i1 := i;
       end;
   end;
   if i1>0 then // point le plus proche
      grandeurToEcran(voies[voie1].valeurInt[i1],
                      voies[voie2].valeurInt[i1],xi,yi);
   x := round((xi-Bx)/Ax);
   y := round((yi-By)/Ay);
   vx := agilent.ValeurVolt(voie1,x);
   vy := agilent.ValeurVolt(voie2,y);
end end;

procedure TgrapheMult.Tri;
// Tri Shell Meyer-Baudoin p 456
// origine vecteur en 0
var
   cle: double;
   i, j: integer;
   v: indiceVoieMult;
   SauvePoint: array[indiceVoieMult] of double;
const
   deuxPi = 360.0;
begin
   i := 1;
   while (i < NbrePoints) do begin
      for v := voieF to VoieGain do
         sauvePoint[v] := voies[v].valeur[i];
      cle := voies[voieF].valeur[i];
      j := pred(i);
      while (j >= 0) and (voies[voieF].valeur[j] > cle) do begin
         for v := voieF to VoieGain do
            voies[v].valeur[succ(j)] := voies[v].valeur[j];
         Dec(j);
      end;
      for v := voieF to VoieGain do
         voies[v].valeur[succ(j)] := sauvePoint[v];
      Inc(i);
   end;
   with voies[voiePhase] do if  (nomUnite = '') then
        for i := 1 to pred(NbrePoints) do
                Valeur[i] := Valeur[i] - round(
                   (Valeur[i] - Valeur[pred(i)]) / DeuxPi) * DeuxPi;
end; // tri

procedure TgrapheMult.EnregistrePointCourant;
var
   i: indiceVoieMult;
begin
   for i := voieF to VoieGain do
      with voies[i] do if active then begin
         valeur[NbrePoints] := valeurCourante;
         stat.reset;
         incertitude[NbrePoints] := incertitudeCourante;
         if i<>voieF then
            TraceCroix(voies[voieF].valeurCourante,valeurCourante,i);
      end;
   Inc(NbrePoints);
end;

procedure TgrapheMult.TraceCroix(xr, yr: double; v: indiceVoieMult);
var
   x, y: integer;
begin
   x  := voies[voieF].GrandeurToEcran(xr);
   y := voies[v].grandeurToEcran(yr);
   PaintBox.Canvas.Pen.color := voies[v].color;
   PaintBox.Canvas.Brush.color := voies[v].color;
   LineG(x-dimX,y,x+dimX,y);
   LineG(x,y-dimY,x,y+dimY);
   // PaintBox.Canvas.Ellipse(x-dimX,y-dimY,x+dimX,y+dimY);
end;


procedure TgrapheMult.DrawG(avecCurseur : boolean);

procedure AfficheCurseur;

   procedure TraceLoc(v: indiceVoieMult);
   var
      x, y: integer;
   begin
      x := voies[voieF].posCurseur;
      y := voies[v].posCurseur;
      if y < 0 then exit;
      with PaintBox.Canvas do begin
      Pen.color := voies[v].color;
      Brush.color := voies[v].color;
      MoveTo(x - 2*dimX, y);
      LineTo(x + 2*dimX, y);
      MoveTo(x, y - 2*dimY);
      LineTo(x, y + 2*dimY);
      end;
   end;

var
   v: indiceVoieMult;
begin
   if ErreurLectureGlb then exit;
   PaintBox.canvas.pen.Width := 3;
   with voies[voieF] do
      posCurseur := GrandeurToEcran(valeurCourante);
   for v := succ(voieF) to VoieGain do
      with voies[v] do if active then begin
      PosCurseur := grandeurToEcran(valeurCourante);
      TraceLoc(v);
   end;
   PaintBox.canvas.pen.Width := 1;
end;

var
   j: indiceVoieMult;
   i: integer;
begin
   LimiteCourbe := paintBox.clientRect;
   with PaintBox.canvas do begin
      font.Name := 'Calibri';
      font.height := 12;
      Pen.Mode := pmCopy;
      brush.color := clBlack;
      font.color  := couleurAxe;
      FillRect(clipRect);
      brush.style := bsSolid;
      pen.Width := 1;
      pen.color := couleurAxe;
   end;
   with voies[voieF], LimiteCourbe do begin
      dimY := (bottom - top) div 128;
      dimX := (right - left) div 128;
      if dimY<dimX then dimY := dimX else dimX := dimY;
      if axeLog then begin
         if mini<1 then mini := 1;
         if maxi<10*mini then maxi := 10*mini;
         A := (right - left) / log10(Maxi / Mini);
         B := left - log10(Mini) * A;
      end
      else begin
         A := (right - left) / (Maxi - Mini);
         B := left - Mini * voies[voieF].A;
      end;
   end;
   PaintBox.canvas.Pen.style := psSolid;
   for j := succ(voieF) to VoieGain do
      with LimiteCourbe, voies[j] do begin
      if axeLog then begin
         if miniLog<1e-4 then miniLog := 1e-4;
         if maxi<10*miniLog then maxi := 10*miniLog;
         A := (top - bottom) / log10(Maxi / MiniLog);
         B := bottom - log10(MiniLog) * A;
      end
      else begin
         A := (top - bottom) / (Maxi - Mini);
         B := bottom - Mini * A;
      end;
      end;
   if quadrillage then drawAxis;
   for i := 0 to pred(NbrePoints) do
      for j := succ(voieF) to VoieGain do
         if voies[j].active then
         TraceCroix(voies[voieF].valeur[i],voies[j].valeur[i],j);
   for j := voieF to VoieGain do
       voies[j].posCurseur := -1;
   if avecCurseur then afficheCurseur;
end; // drawG

procedure TgrapheOsc.DrawG;
var erreur : Terreur;

procedure TraceBaseDeTemps;
var pas,j,xi,yi,i,z : integer;
    erreurPrec : Terreur;
begin
   erreurPrec := erNo;
   pas := (screen.Width div agilent.NbrePoints) + 1;
   for j := voie1 to voie2 do if agilent.voies[j].active then begin
       PaintBox.canvas.pen.color := couleurAgilent[j];
       grandeurToEcran(0,agilent.voies[j].valeurInt[0],xi,yi);
       PaintBox.canvas.moveTo(xi,yi);
       i := pas;
       while i<agilent.NbrePoints do begin
           z := agilent.voies[j].valeurInt[i];
           erreur := agilent.isOK(z);
           grandeurToEcran(i,z,xi,yi);
           if erreur=erNo then
              if erreurPrec=erNo
                 then PaintBox.canvas.lineTo(xi,yi)
                 else PaintBox.canvas.moveTo(xi,yi)
           else begin
               case erreur of
                    erLow : yi := PaintBox.canvas.clipRect.bottom+1;
                    erHigh : yi := PaintBox.canvas.clipRect.top-1;
                    erHole : yi := PaintBox.canvas.clipRect.top+1;
                    else yi := -1;
               end;
               PaintBox.canvas.pixels[xi,yi] := clRed;
           end;
           erreurPrec := erreur;
           inc(i,pas);
      end;
   end;
end;

procedure TraceXY;
var xi,yi,i,x,y : integer;
    erreurPrec : Terreur;
begin
       PaintBox.canvas.pen.color := couleurXY;
       erreurPrec := erHole;
       for i := agilent.debut to agilent.fin do begin
           x := agilent.voies[voie1].valeurInt[i];
           erreur := agilent.isOK(x);
           if erreur=erNo
             then begin
               y := agilent.voies[voie2].valeurInt[i];
               erreur := agilent.isOK(y);
             end
             else y := 0; // pour le compilateur
           if erreur=erNo then begin
              grandeurToEcran(x,y,xi,yi);
              if erreurPrec=erNo
                 then PaintBox.canvas.lineTo(xi,yi)
                 else PaintBox.canvas.moveTo(xi,yi);
           end;
           erreurPrec := erreur;
      end;
      xi := agilent.zero(voie1);
      yi := agilent.zero(voie2);
      with PaintBox.canvas do begin
      Pen.Color := clWhite;
      Pen.style := psDot;
      MoveTo(xi,PaintBox.ClientRect.top);
      MoveTo(xi,PaintBox.ClientRect.bottom);
      MoveTo(PaintBox.ClientRect.left,yi);
      MoveTo(PaintBox.ClientRect.right,yi);
      Pen.style := psSolid;
      end;
end;

procedure TraceXYold(k : integer);
var xi,yi,i,x,y : integer;
    erreurPrec : Terreur;
begin with TdataAgilent(oldCourbes[k]) do begin
       PaintBox.canvas.pen.color := CouleurPale(couleurXY,k);
       erreurPrec := erHole;
       for i := debut to fin do begin
           x := voies[voie1].valeurInt[i];
           erreur := isOK(x);
           if erreur=erNo
              then begin
                y := voies[voie2].valeurInt[i];
                erreur := isOK(y);
              end
              else y := 0; // pour le compilateur
           if erreur=erNo then begin
              grandeurToEcran(x,y,xi,yi);
              if erreurPrec=erNo
                 then PaintBox.canvas.lineTo(xi,yi)
                 else PaintBox.canvas.moveTo(xi,yi);
           end;
           erreurPrec := erreur;
      end;
end end;

var i : integer;
begin
   with PaintBox.canvas do begin
     Pen.Mode  := pmCopy;
     brush.color := clBlack;
     font.color  := couleurAxe;
     FillRect(clipRect);
     brush.style := bsSolid;
     pen.Width := 1;
     pen.color := couleurAxe;
   end;
   if agilent.NbrePoints<16 then exit;
   with PaintBox.clientRect do begin
        AX := (right - left) / agilent.NbrePoints;
        BX := left;
   end;
   PaintBox.canvas.Pen.style := psSolid;
   with PaintBox.ClientRect do if agilent.infiniVision
       then begin
           case agilent.codage of
              varByte : AY := (top-bottom) / 256;
              varWord : AY := (top-bottom) / 256 / 256;
           end;
           BY := bottom;
      end
      else begin
          AY := (bottom-top) / 256;
          BY := top;
      end;
   if modeXY then with PaintBox.clientRect do if agilent.infiniVision
       then begin
           case agilent.codage of
              varByte : AX := (right-left) / 256;
              varWord : AX := (right-left) / 256 / 256;
           end;
           BX := left;
      end
      else begin
          AX := (left-right) / 256;
          BX := right;
      end;
   if modeXY
      then begin
          traceXY;
          for i := 0 to oldCourbes.Count - 1 do
              traceXYold(i);
      end
      else traceBaseDeTemps;
end; // TgrapheOsc.draw

constructor TgrapheOsc.Create;
begin
    oldCourbes := Tlist.Create;
    Ax := 1;Bx := 0;
    Ay := 1;By := 0;
end;

destructor TgrapheOsc.Destroy;
var B : integer;
    AData : TdataAgilent;
begin
  for B := 0 to (oldCourbes.Count - 1) do begin
     AData := TdataAgilent(oldCourbes[B]);
     AData.free;
  end;
  oldCourbes.free;
  inherited
end;

constructor TgrapheMult.Create;
var
   j: indiceVoieMult;
begin
   inherited Create;
   for j := voieF to voieGain do
      voies[j] := Tvoie.Create;
   voies[voieF].color := clBlack;
   voies[voie1].color := clYellow;
   voies[voie2].color := clLime;
   voies[voiePhase].color := clFuchsia;
   voies[voieGain].color := clAqua;
   voies[voieF].axeLog := True;
   voies[voieGain].active := False;
   quadrillage := True;
   erreurLectureGlb := False;
   NbrePoints := 0;
   dimY := 4;
   dimX := 2;
end;

destructor TgrapheMult.Destroy;
var
   j: indiceVoieMult;
begin
   for j := voieF to voieGain do voies[j].free;
   inherited Destroy;
end;

procedure TgrapheMult.LineG(x1, y1, x2, y2: integer);
begin
   PaintBox.canvas.moveTo(x1, y1);
   PaintBox.canvas.lineTo(x2, y2);
end;

procedure TgrapheMult.DrawAxis;

   procedure traceAxeLogx;
   var
      i, j: integer;
      xs : integer;
      intermediaire: boolean;
   begin with PaintBox.canvas, limiteCourbe, voies[voieF] do begin
         pen.color := CouleurAxe;
         Intermediaire := log10(Maxi / Mini) <= 4;
         for i := trunc(log10(Mini)) to ceil(log10(Maxi)) do begin
            xs := grandeurToEcran(power(10,i));
            pen.style := psSolid;
            LineG(xs, top, xs, bottom);
            pen.style := psDot;
            if intermediaire then
               for j := 2 to 9 do begin
                  xs := grandeurToEcran(power(10,i + log10(j)));
                  LineG(xs, top, xs, bottom);
               end;{for j}
         end; {for i}
   end end; // TraceAxeLogx

   procedure traceAxedB;
   var
      i : integer;
      ys,dy : integer;
   begin with PaintBox.canvas, limiteCourbe, voies[voieGain] do begin
         pen.color := color;
         pen.Style := psDash;
         font.color := color;
         dy := textHeight('A') div 2;
         for i := trunc(log10(MiniLog)) to ceil(log10(Maxi)) do begin
            ys := grandeurToEcran(power(10,i));
            LineG(left, ys, right, ys);
            TextOut(left + 3, ys-dy, intToStr(20*i)+'dB');
         end; {for i}
   end end; // TraceAxedB

   procedure traceAxeX;
   var
      i:  integer;
      xs: integer;
   begin with PaintBox.canvas, limiteCourbe do begin
         pen.color := CouleurAxe;
         pen.Style := psDot;
         for i := 0 to 10 do begin
            xs := round(left + i * (right - left) div 10);
            LineG(xs, top, xs, bottom);
         end; {for i}
   end end; // TraceAxeX

   procedure traceAxePhase;
   var
      i:  integer;
      ys,xt,dy: integer;
      y:  double;
   begin with PaintBox.canvas, limiteCourbe do begin
         pen.color := voies[voiePhase].color;
         pen.Style := psSolid;
         font.color := voies[voiePhase].color;
         xt := right-4*textWidth('A');
         dy := textHeight('A') div 2;
         for i := -2 to 2 do begin
            y  := i * 90;
            if i =0
               then pen.Style := psSolid
               else pen.Style := psDot;
            ys := voies[voiePhase].grandeurToEcran(y);
            LineG(left, ys, xt-8, ys);
            TextOut(xt, ys-dy, intToStr(round(y)));
         end;
   end end; // TraceAxePhase

   procedure traceAxeGain;
   var
      i:  integer;
      ys,dy: integer;
      y,deltay:  double;
   begin with PaintBox.canvas, limiteCourbe do begin
         pen.color := CouleurAxe;
         pen.Style := psDash;
         font.color := voies[voieGain].color;
         with voies[voieGain] do deltaY := (maxi-mini)/10;
         dy := textHeight('A') div 2;
         for i := 0 to 10 do begin
            y  := i*deltaY;
            ys := voies[voieGain].grandeurToEcran(y);
            LineG(left, ys, right, ys);
            TextOut(left, ys-dy, formatCourt(y,''));
         end;
   end end; // TraceAxeGain

    procedure traceAxeY;
    var
      i:  integer;
      ys: integer;
    begin with PaintBox.canvas, limiteCourbe do begin
      pen.color := CouleurAxe;
      pen.Style := psDot;
      for i := 0 to 10 do begin
         ys := round(top + i * (bottom - top) div 10);
         LineG(left, ys, right, ys);
      end;
    end end;

begin with PaintBox.canvas, LimiteCourbe do begin
      if voies[voieF].axeLog
         then traceAxeLogX
         else traceAxeX;
      TextOut(right + 2 * font.Height, top + 2, voies[voieF].nom);
      pen.Style := psSolid;
      if voies[voieGain].active
         then if voies[voieGain].axeLog
            then traceAxedB
            else traceAxeGain
         else traceAxeY;
      if voies[voiePhase].active then
         traceaxePhase;
end end; // DrawAxis

procedure TgrapheMult.SaveData(const NomFichier: string);
var
   fichier: textFile;
   formatRW3: boolean;

   Procedure sautDeLigne;
   begin
       write(Fichier,crLF)
   end;

   procedure EcritPage;
   var
      j: integer;
      i: indiceVoieMult;
   begin
      if formatRW3 then begin
         writeln(fichier, SymbReg, '1 PAGE COMMENT');
         writeln(fichier, '');
         writeln(fichier, SymbReg2, NbrePoints, ' VALEUR VAR');
         for j := 0 to pred(NbrePoints) do begin
            Write(fichier, FloatToStrPoint(voies[voieF].valeur[j]), ' ');
            for i := succ(voieF) to VoieGain do if voies[i].active then
               Write(fichier, FloatToStrPoint(voies[i].valeur[j]), ' ');
            writeln(fichier);
         end;
      end
      else begin
         for j := 0 to pred(NbrePoints) do begin
            Write(fichier, FloatToStr(voies[voieF].valeur[j]));
            for i := succ(voieF) to VoieGain do if voies[i].active then
               Write(fichier, separeValue, FloatToStr(voies[i].valeur[j]));
            sautDeLigne;
         end;
      end;
   end; // ecritPage

   procedure ecritEnTete;
   var
      j: indiceVoieMult;
      N: integer;
   begin
      if formatRW3 then begin
         N := voiePhase-voieF;
         if voies[voiePhase].active then inc(N);
         if voies[voieGain].active then inc(N);
         writeln(fichier, 'EVARISTE REGRESSI WINDOWS 1.0');
         writeln(fichier, symbReg, N, ' NOM VAR');
         for j := voieF to VoieGain do if voies[j].active then
            writeln(fichier, unicodeVersRW3(voies[j].nom));
         writeln(fichier, symbReg, N, ' UNITE VAR');
         for j := voieF to VoieGain do if voies[j].active then
            writeln(fichier, voies[j].nomUnite);
         writeln(fichier, symbReg, N, ' SIGNIF VAR');
         for j := voieF to VoieGain do if voies[j].active then
            writeln(fichier, unicodeVersRW3(voies[j].signif));
         writeln(fichier, symbReg, '1 TRIGO');
         writeln(fichier, Ord(True));
         writeln(fichier, symbReg, '1 ACQUISITION');
         writeln(fichier, Application.ExeName);
      end
      else if not OptionsCSVForm.dataOnly then begin
            Write(fichier, voies[voieF].nom);
            for j := succ(voieF) to VoieGain do if voies[j].active then
                Write(fichier, separeValue, voies[j].nom);
            sautDeLigne;
            Write(fichier, voies[voieF].nomUnite);
            for j := succ(voieF) to VoieGain do if voies[j].active then
               Write(fichier, separeValue, voies[j].nomUnite);
            sautDeLigne;
      end;
   end; // ecritEnTete

var extension : string;
begin // saveData
   extension := AnsiUpperCase(ExtractFileExt(NomFichier));
   FormatRW3 := extension = '.RW3';
   AssignFile(fichier, NomFichier);
   Rewrite(fichier);
   ecritEnTete;
   tri;
   ecritPage;
   CloseFile(fichier);
end; // saveData

procedure Tvoie.SetIncertitude;  // en cart-type ? donc LSB/rac(12)
var
   deltaT, deltaV: double;
begin
   stat.AjouteValeur(valeurCourante);
   deltaT := baseDeTemps / 1024 / sqrt(12); // 1024 points
   deltaV := FullScale / 256 / sqrt(12); //  8 bits
   if isNan(valeurCourante) then incertitudeCourante := 0
      else begin
      case calculIncertitude of
         ciFreq: incertitudeCourante  := valeurCourante / 1e4 + sqr(valeurCourante) * DeltaT;
         // 0,01% lecture + 0,1% screenWidth
         ciTemps: incertitudeCourante := valeurCourante / 1e4 + DeltaT;
         ciTension: incertitudeCourante := DeltaV;
         // calibre : 2% pleine chelle
         // offset : 0,5% offset + 1% pleine chelle
         ciDuty: incertitudeCourante  := (1 + valeurCourante) * DeltaT * FrequenceCourante;
         ciPhase: incertitudeCourante :=
               (360 + valeurCourante) * DeltaT * FrequenceCourante;
         ciTensionEff: incertitudeCourante := DeltaV / 2; // effet de la moyenne
         else incertitudeCourante := 0;
      end;
      case incertitudeComposee of
           iConstructeur : ;
           icMoyenne : incertitudeCourante := sqrt(sqr(incertitudeCourante)+sqr(stat.t95));
           icSigma : incertitudeCourante := sqrt(sqr(incertitudeCourante)+sqr(stat.sigma));
      end;
      end;
end;

procedure Tvoie.SetIncertitudeDSOX;  // en cart-type
const IncertTemps = 30E-6;
var  pas : double;
begin
   incertitudeCourante := 0;
   stat.AjouteValeur(valeurCourante);
   pas := BaseDeTemps/5000;
   if isNan(valeurCourante) then
      else begin
      case calculIncertitude of
         ciNone: ;
         ciFreq: incertitudeCourante  := valeurCourante*incertTemps + sqr(valeurCourante) * pas;
         ciTemps: incertitudeCourante := valeurCourante*incertTemps + pas;
         ciTension: incertitudeCourante := incertitudeType;
         ciDuty: incertitudeCourante  := (1 + valeurCourante) * pas * FrequenceCourante;
         ciPhase: incertitudeCourante :=
               (360 + valeurCourante) * pas * FrequenceCourante;
         ciTensionEff: incertitudeCourante := incertitudeType;
         ciGain : incertitudeCourante := 0;
      end;
      case incertitudeComposee of
           iConstructeur : ;
           icMoyenne : incertitudeCourante := sqrt(sqr(incertitudeCourante)+sqr(stat.t95));
           icSigma : incertitudeCourante := sqrt(sqr(incertitudeCourante)+sqr(stat.sigma));
      end;
      end;
end;

function Tvoie.StrValeur(nombre: double): string;
var
   coeff: coeffUnite;
   Negatif: boolean;
   tampon: string;
   i, N: integer;
begin
   if isNan(nombre)
   then Result := ' ? '
   else begin
      if nomUnite = ''
      then result := FloatToStrF(nombre, ffFixed, 3, 1) + ' '
      else if nomUnite = 'dB'
      then Result := FloatToStrF(20*log10(nombre), ffFixed, 3, 1)+ ' dB'
      else begin
         negatif := nombre < 0;
         nombre := abs(nombre);
         coeff := nulle;
         while (nombre >= 999) and (coeff < infini) do begin
            nombre := nombre / 1000;
            Inc(coeff);
         end; { nombre<1000 ou infini }
         while (nombre < 0.999) and (coeff > zero) do begin
            nombre := nombre * 1000;
            Dec(coeff);
         end; { nombre>=1 ou zero }
         if coeff in [infini, zero]
            then tampon := '0 '
            else begin
            tampon := FloatToStrF(nombre, ffGeneral, 3, 0);
            i := 1;
            N := 0;
            while (N < 4) and (i < length(tampon)) do begin
               if charInSet(tampon[i],['0'..'9']) then
                  Inc(N);
               Inc(i);
            end;
            // N=precision ou fin de chaine
            if N = 4 then tampon := copy(tampon, 1, i - 1);
            if negatif then tampon := '-' + tampon;
            tampon := tampon + ' ';
            if coeff <> nulle then
               if nomUnite<>''
                  then tampon := tampon + symboleUnite[coeff]
                  else tampon := tampon + expUnite[coeff];
         end;
         Result := tampon + nomUnite;
      end;
   end;
end;

function Tvoie.AffVoltmetre: string;
begin
   Result := nom + '=' + strValeur(valeurCourante);
end;

end.

