{
  LibUSB.DLL's Delphi declarations.
  Modify from PowerSwitch's LIBUSB.pas.
  This file direct access LibUSB.dll, not like PowerSwitch use USBLibExport.dll.
  Date:    2006-Nov-15
  Version: 1.0
  Author:  Shaoziyang
           mailto:shaoziyang@gmail.com
           http://shaoziyang.bloger.com.cn
  Usage:
  1. add LibUSB to uses section;
  2. define a global value as TLIBUSB class;
  3. use USBdevInit to initial user USB device;
  4. set user's PID and VID;
  5. use USBdevOpen to open USB device;
  6. use USBdevRead and USBdevWrite to access user's USB device;
  7. use USBdevClose to close device when exit;
}

unit LibUSB;

interface

uses
  Windows, SysUtils, Classes, ActiveX, VisaComLib_TLB,
  jmmUtil, variants, registry, generics.collections; // HPserie

type
  Tusb = (oLibUSB,oVisaCom,oRigol); // oRS232
  CharArray = array of byte;

var
  DLLLoaded: Boolean = False;
  DLLHandle: THandle;
  ErrorMode: Integer;
  ErrorVisa : string;
  rm : IResourceManager3;
{$IFDEF Debug}
      fichierDebug : textFile;
{$ENDIF}

const
  LIBUSB_PATH_MAX = 512;

  USB_OK = 0; // 0 = success from functions < 0 is failed

  { Device and/or Interface Class codes }
  USB_CLASS_PER_INTERFACE = 0; { for DeviceClass }
  USB_CLASS_AUDIO = 1;
  USB_CLASS_COMM = 2;
  USB_CLASS_HID = 3;
  USB_CLASS_PRINTER = 7;
  USB_CLASS_MASS_STORAGE = 8;
  USB_CLASS_HUB = 9;
  USB_CLASS_DATA = 10;
  USB_CLASS_VENDOR_SPEC = $FF;

  // Descriptor types
  USB_DT_DEVICE = $01;
  USB_DT_CONFIG = $02;
  USB_DT_STRING = $03;
  USB_DT_INTERFACE = $04;
  USB_DT_ENDPOINT = $05;

  USB_DT_HID = $21;
  USB_DT_REPORT = $22;
  USB_DT_PHYSICAL = $23;
  USB_DT_HUB = $29;

  // Descriptor sizes per descriptor type
  USB_DT_DEVICE_SIZE = 18;
  USB_DT_CONFIG_SIZE = 9;
  USB_DT_INTERFACE_SIZE = 9;
  USB_DT_ENDPOINT_SIZE = 7;
  USB_DT_ENDPOINT_AUDIO_SIZE = 9; // Audio extension
  USB_DT_HUB_NONVAR_SIZE = 7;

  // Endpoint descriptor
  USB_MAXENDPOINTS = 32;

  USB_ENDPOINT_ADDRESS_MASK = $0F; // in bEndpointAddress
  USB_ENDPOINT_DIR_MASK = $80;

  USB_ENDPOINT_TYPE_MASK = $03; // in bmAttributes
  USB_ENDPOINT_TYPE_CONTROL = 0;
  USB_ENDPOINT_TYPE_ISOCHRONOUS = 1;
  USB_ENDPOINT_TYPE_BULK = 2;
  USB_ENDPOINT_TYPE_INTERRUPT = 3;

  // Interface descriptor
  USB_MAXINTERFACES = 32;

  USB_MAXALTSETTING = 128; // Hard limit

  // Configuration descriptor information..
  USB_MAXCONFIG = 8;

  // Standard requests
  USB_REQ_GET_STATUS = $00;
  USB_REQ_CLEAR_FEATURE = $01;
  // $02 is reserved
  USB_REQ_SET_FEATURE = $03;
  // $04 is reserved
  USB_REQ_SET_ADDRESS = $05;
  USB_REQ_GET_DESCRIPTOR = $06;
  USB_REQ_SET_DESCRIPTOR = $07;
  USB_REQ_GET_CONFIGURATION = $08;
  USB_REQ_SET_CONFIGURATION = $09;
  USB_REQ_GET_INTERFACE = $0A;
  USB_REQ_SET_INTERFACE = $0B;
  USB_REQ_SYNCH_FRAME = $0C;

  USB_TYPE_STANDARD = ($00 shl 5);
  USB_TYPE_CLASS = ($01 shl 5);
  USB_TYPE_VENDOR = ($02 shl 5);
  USB_TYPE_RESERVED = ($03 shl 5);

  USB_RECIP_DEVICE = $00;
  USB_RECIP_INTERFACE = $01;
  USB_RECIP_ENDPOINT = $02;
  USB_RECIP_OTHER = $03;

  // Various libusb API related stuff
  USB_ENDPOINT_IN = $80;
  USB_ENDPOINT_OUT = $00;

  // Error codes
  USB_ERROR_BEGIN = 500000;

type
  // All standard descriptors have these 2 fields in common
  usb_descriptor_header = packed record
    bLength,
      bDescriptorType: byte;
  end;
  // String descriptor
  usb_string_descriptor = packed record
    bLength,
      bDescriptorType: byte;
    wData: packed array[0..0] of word;
  end;
  usb_hid_descriptor = packed record
    bLength,
      bDescriptorType: byte;
    bcdHID: word;
    bCountryCode,
      bNumDescriptors: byte;
  end;
  // Endpoint descriptor
  pusb_endpoint_descriptor = ^usb_endpoint_descriptor;
  usb_endpoint_descriptor = packed record
    bLength,
      bDescriptorType,
      bEndpointAddress,
      bmAttributes: byte;

    wMaxPacketSize: word;

    bInterval,
      bRefresh,
      bSynchAddress: byte;

    extra: PByte; // Extra descriptors
    extralen: integer;
  end;
  // pascal translation of C++ struc
  TArray_usb_endpoint_descriptor = packed array[0..65535] of usb_endpoint_descriptor;
  PArray_usb_endpoint_descriptor = ^TArray_usb_endpoint_descriptor;

  // Interface descriptor
  pusb_interface_descriptor = ^usb_interface_descriptor;
  usb_interface_descriptor = packed record
    bLength,
      bDescriptorType,
      bInterfaceNumber,
      bAlternateSetting,
      bNumEndpoints,
      bInterfaceClass,
      bInterfaceSubClass,
      bInterfaceProtocol,
      iInterface: byte;

    endpoint: PArray_usb_endpoint_descriptor;

    extra: PByte; // Extra descriptors
    extralen: integer;
  end;
  // pascal translation of C++ struc
  TArray_usb_interface_descriptor = packed array[0..65535] of usb_interface_descriptor;
  PArray_usb_interface_descriptor = ^TArray_usb_interface_descriptor;

  pusb_interface = ^usb_interface;
  usb_interface = packed record
    altsetting: PArray_usb_interface_descriptor;
    num_altsetting: integer;
  end;
  // pascal translation of C++ struc
  TArray_usb_interface = packed array[0..65535] of usb_interface;
  PArray_usb_interface = ^TArray_usb_interface;

  // Configuration descriptor information..
  pusb_config_descriptor = ^usb_config_descriptor;
  usb_config_descriptor = packed record
    bLength,
      bDescriptorType: byte;

    wTotalLength: word;

    bNumInterfaces,
      bConfigurationValue,
      iConfiguration,
      bmAttributes,
      MaxPower: byte;

    iinterface: PArray_usb_interface;

    extra: PByte; // Extra descriptors
    extralen: integer;
  end;
  // pascal translation of C++ struc
  TArray_usb_config_descriptor = packed array[0..65535] of usb_config_descriptor;
  PArray_usb_config_descriptor = ^TArray_usb_config_descriptor;

  // Device descriptor
  usb_device_descriptor = packed record
    bLength,
      bDescriptorType: byte;
    bcdUSB: word;

    bDeviceClass,
      bDeviceSubClass,
      bDeviceProtocol,
      bMaxPacketSize0: byte;

    idVendor,
      idProduct,
      bcdDevice: word;

    iManufacturer,
      iProduct,
      iSerialNumber,
      bNumConfigurations: byte;
  end;

  usb_ctrl_setup = packed record
    bRequestType,
      bRequest: byte;
    wValue,
      wIndex,
      wLength: word;
  end;

  pusb_bus = ^usb_bus;
  pusb_device = ^usb_device;
  usb_device = packed record
    next, prev: pusb_device;
    filename: packed array[0..LIBUSB_PATH_MAX - 1] of AnsiChar;
    bus: pusb_bus;
    descriptor: usb_device_descriptor;
    config: PArray_usb_config_descriptor;

    dev: pointer; // Darwin support
  end;

  usb_bus = packed record
    next, prev: pusb_bus;
    dirname: packed array[0..LIBUSB_PATH_MAX - 1] of AnsiChar;
    devices: pusb_device;
    location: longint;
  end;

  pusb_dev_handle = ^usb_dev_handle;
  usb_dev_handle = packed record
    fd: Integer;
    bus: pusb_bus;
    dev: pusb_device;
    cfg: Integer;
    intfac: Integer;
    altset: Integer;
    impl_info: Pointer;
  end;

  // Version information, Windows specific
  pusb_version = ^usb_version;
  usb_version = packed record
    dllmajor,
      dllminor,
      dllmicro,
      dllnano: integer;
    drivermajor,
      driverminor,
      drivermicro,
      drivernano: integer;
  end;

var
  //LibUSB functions declarations
usb_init : procedure; cdecl ;
usb_set_debug : procedure(level: Integer); cdecl ;
usb_devices : function(dev: pusb_dev_handle): pusb_device; cdecl ;
usb_find_busses : function: Integer; cdecl ;
usb_find_devices : function: Integer; cdecl ;
usb_get_busses : function: pusb_bus; cdecl ;

usb_get_version : function: pusb_version; cdecl ;
usb_open : function(dev: pusb_device): pusb_dev_handle; cdecl ;
usb_close : function(dev: pusb_dev_handle): Integer; cdecl ;
usb_get_string : function(dev: pusb_dev_handle; index, langid: Integer;
                        buf: array of AnsiChar; len: Integer): Integer; cdecl ;
usb_get_string_simple : function(dev: pusb_dev_handle; index: Integer;
                        buf: array of AnsiChar; len: Integer): Integer; cdecl ;

usb_get_descriptor_by_endpoint : function(dev: pusb_dev_handle; ep: Integer;
              tp: Byte; index: Byte;var buf; size: Integer): Integer; cdecl ;
usb_get_descriptor : function(dev: pusb_dev_handle; tp, index: Byte;
              var buf; size: Integer): Integer; cdecl ;
usb_control_msg : function(dev: pusb_dev_handle;
                  requesttype, request, value, index: Integer;
                  var bytes; size, timeout: Integer): Integer; cdecl ;
usb_bulk_write : function(dev: pusb_dev_handle; ep: Integer; bytes: PAnsiChar;
                        size, timeout: Integer): Integer; cdecl ;
usb_bulk_read : function(dev: pusb_dev_handle; ep: Integer; bytes: PAnsiChar;
                       size, timeout: Integer): Integer; cdecl ;
usb_interrupt_write : function(dev: pusb_dev_handle; ep: Integer;
                             bytes: PAnsiChar; size, timeout: Integer): Integer; cdecl ;
usb_interrupt_read : function(dev: pusb_dev_handle; ep: Integer;
                            bytes: PAnsiChar; size, timeout: Integer): Integer; cdecl ;

usb_set_configuration : function(dev: pusb_dev_handle;
                          configuration: Integer): pusb_device; cdecl ;
usb_claim_interface : function(dev: pusb_dev_handle;
                          interfaces: Integer): Integer; cdecl ;
usb_release_interface : function(dev: pusb_dev_handle;
                          interfaces: Integer): Integer; cdecl ;
usb_set_altinterface : function(dev: pusb_dev_handle;
                          alternate: Integer): Integer; cdecl ;
usb_resetep : function(dev: pusb_dev_handle; ep: Word): Integer; cdecl ;
usb_clear_halt : function(dev: pusb_dev_handle; ep: Word): Integer; cdecl ;
usb_reset : function(dev: pusb_dev_handle): Integer; cdecl ;

const maxLen = 256;

type
  TLIBUSB = class
  // Visa COM
    io : IMessage;
    sess : IVisaSession;
    bus: pusb_bus;
    dev: pusb_device;
    handle: pusb_dev_handle;
    VID, PID: word;
    version: pusb_version;
    dat: array[0..maxLen] of byte;
    usb : Tusb;
    FVisa_open : boolean;
    reponseVisa : HResult;
    identificateur : string;
    procedure setOpen(avalue : boolean);
    function getOpen : boolean;
    public
    NomVisaCom : string;
    property open : boolean read getOpen write setOpen;
    constructor create(avid,apid : word);
    procedure flush;
    function readString(var reponse : AnsiString) : integer;
    function readWord(var count : integer;var reponse : CharArray) : integer;
    function writeString(str : AnsiString) : boolean;
    function getString(command: AnsiString) : AnsiString;
    destructor Destroy;override;
  end;

  TDataVisaCOM = class
     Nom : string;
     CodeConstructeur,CodeAppareil : integer;
  end;

procedure setNomVisa;

var
    ListeVisaCom : TList<TDataVisaCom>;

implementation

const
  crLF = #10;
  crCR = #13;


procedure setNomVisa;
var
    pBuffer: PSafeArray;
    i, LBound, HBound, j : Integer;
    data : wideString;
    dataStr : string;
    posCode1,posCode2,posCode3 : integer;
    strConstructeur,strAppareil : string;
    codeApp,codeConst : integer;
    appareil : TDataVisaCom;
    existe : boolean;
begin
    try
{$IFDEF Debug}
      append(fichierDebug);
      writeln(fichierDebug,'Recherche USB');
      system.flush(fichierDebug);
      closeFile(fichierDebug);
{$ENDIF}

    ListeVisaCom.clear;
    if rm.FindRsrc('USB?*',pBuffer)=0 then begin
         SafeArrayGetLBound(pBuffer, 1, LBound);
         SafeArrayGetUBound(pBuffer, 1, HBound);
         for I := LBound to HBound do
             if SafeArrayGetElement(pBuffer,i,data)=0 then begin
                dataStr := string(data); // usb0::2391::1031::MY43002410::INSTR
{$IFDEF Debug}
      append(fichierDebug);
      writeln(fichierDebug,'USB? ',dataStr);
      system.flush(fichierDebug);
      closeFile(fichierDebug);
{$ENDIF}

                poscode1 := pos('::',dataStr);
                codeApp := 0;codeConst := 0;
                if poscode1>0 then begin
                   poscode2 := pos('::',dataStr,poscode1+2);
                   if (poscode2>0) then begin
                     poscode3 := pos('::',dataStr,poscode2+2);
                     strConstructeur := copy(data,poscode1+2,poscode2-poscode1-2);
                     codeConst := strToInt(strConstructeur);
                     // 2391 = Agilent
                     if (poscode3>0) then begin
                     strAppareil := copy(data,poscode2+2,poscode3-poscode2-2);
                     codeApp := strToInt(strAppareil);
                     existe := false;
                     for j := 0 to ListeVisaCOM.count-1 do begin
                       existe := (codeApp=ListeVisaCom[j].CodeAppareil) and
                                 (codeConst=ListeVisaCom[j].CodeConstructeur);
                       if existe then break;
                     end;
                     if existe then continue;
                     end;
                   end;
                end;
                Appareil := TdataVisaCom.Create;
                Appareil.Nom := data;
                Appareil.codeAppareil := codeApp;
                Appareil.codeConstructeur := codeConst;
                ListeVisaCom.Add(appareil);
             end;
     end;
     except
     end;
{$IFDEF Debug}
      append(fichierDebug);
      writeln(fichierDebug,'Nbre:',ListeVisaCom.count);
      system.flush(fichierDebug);
      closeFile(fichierDebug);
{$ENDIF}

end;

constructor TLIBUSB.create(avid,apid : word);
begin
  inherited create;
  dev := nil;
  handle := nil;
  Identificateur := '';
  VID := avid;
  PID := apid;
  Fvisa_open := false;
//  'USB0::0x0957::0x0588::CN49251905::0::INSTR';
//  'USB0::0x0957::0x179B::MY54233244::0::INSTR';
end;

procedure TLIBUSB.setOpen(avalue : Boolean);

Procedure CloseVisa;
begin
   if Fvisa_open then begin
       if io<>nil then io.Close; // ...
       sess := nil; // cause Delphi to release the COM objects
       Fvisa_open := false;
   end;
end;


Procedure OpenVisa;
begin
{$IFDEF Debug}
      append(fichierDebug);
      writeln(fichierDebug,'OpenVisa');
      writeln(fichierDebug,'NomVisa;',NomVisaCom);
      system.flush(fichierDebug);
      closeFile(fichierDebug);
{$ENDIF}
   ErrorVisa := '';
   CloseVisa;

   try

   if (ListeVisaCom.count=0) and (nomVisaCom='') then begin

      ErrorVisa := 'Aucun appareil Visa trouv';
      exit;
   end;
// NomVisaCom <>'' and  (ListeVisaCom.count=0) => on force le nom manuellement
   FVisa_open := rm.Open(nomVisaCom, NO_LOCK, 1000, '', sess)=0;// 1000 = timeout
// Use the resource manager to create a VISA COM Session
   if FVisa_open then begin
      sess.QueryInterface(IID_IMessage, io);
// The IVisaSession interface is very general and does not have string
      io.Clear;
      io.Set_Timeout(5000); // 10000 ?
      usb := oVisaCom;
      Identificateur := string(getString('*IDN?'));
{$IFDEF Debug}
      append(fichierDebug);
      writeln(fichierDebug,'IDN? ',identificateur);
      system.flush(fichierDebug);
      closeFile(fichierDebug);
{$ENDIF}
      if identificateur='' then begin // on ressaye
         WriteString('*CLS');
          // WriteString(':SYST:PRES'); // System:Preset = front panel Default Setup

         sleep(1000);

         Identificateur := string(getString('*IDN?'));
      end;
   end
   else ErrorVisa := 'Problme  l''ouverture de la liaison avec l''oscilloscope';
   except
        ErrorVisa := 'Problme : IO Libraries Suite 17 non install ?';
        FVisa_open := false;
   end;
{$IFDEF Debug}
      append(fichierDebug);
      writeln(fichierDebug,'Error:',ErrorVisa);
      system.flush(fichierDebug);
      closeFile(fichierDebug);
{$ENDIF}
end;

Procedure CloseLibUSB;
begin
  if handle = nil then Exit;
  if usb_close(handle) = 0 then begin
    dev := nil;
    handle := nil;
  end;
end;

Procedure OpenLibUSB;
begin
  if not DLLLoaded then exit;
  usb_init;
  usb_find_busses;
  usb_find_devices;
  bus := usb_get_busses;
  while bus <> nil do begin
    dev := bus.devices;
    while dev <> nil do begin
      if (dev.descriptor.idVendor = VID) and
         (dev.descriptor.idProduct = PID) then begin
        handle := usb_open(dev);
        usb := oLibUsb;
        Identificateur := string(getString('*IDN?'));
        exit;
      end;
      dev := dev.next;
    end;
    bus := bus.next;
  end;
end;

begin
  ErrorVisa := '';
  if avalue then begin
    if (nomVisaCom='') and not open then openLibUSB;
    if not open then openVisa;
  end
  else if open then begin
     if handle<>nil then closeLibUSB;
     closeVisa;
//     if FRS232_open then closeRS232;
  end;
end;

function TLibUSB.getString(command : AnsiString) : AnsiString;
var len : integer;
    buf : ansiString;
begin
      result := '';
      writeString(command);
      len := readString(buf);
      if (len>0) and (buf[len]=crLF) then buf := copy(buf,1,len-1);
      result := buf;
end;

function TLIBUSB.getOpen : Boolean;
begin
   result := Fvisa_open or (handle <> nil)
    // VisaCom or libUsb
end;

function TLIBUSB.readString(var reponse : AnsiString) : integer;

procedure get_VisaCom;
var bufW: wideString;
begin
   reponseVisa := io.ReadString(1024, bufW);
// reponse : S_OK 0
// S_VISA_MAX_CNT 0x40006 : on a lu 1024 caractres
// S_VISA_TERM_CHAR 0x40005 : fin des donnes
   reponse := ansiString(bufW);
{$IFDEF Debug}
      append(fichierDebug);
      writeln(fichierDebug,'#Read:',reponse);
      system.flush(fichierDebug);
      closeFile(fichierDebug);
{$ENDIF}
end;

procedure get_LIBUSB;
var i : integer;
    len : word;
begin
   usb_control_msg(handle, $C0, 0, 0, 0, len, 1, 1000);
// rcupre la longueur disponible de la rponse
   if len=0 then exit;
   if len>maxLen then len := maxLen;
   len := usb_control_msg(handle, $C0, 0, 1, 0, dat, len, 1000);
   for i := 0 to pred(len) do
       reponse := reponse + AnsiChar(dat[i]);
end;

begin
   result := 0;
   reponse := '';
   reponseVisa :=  S_VISA_SUCCESS;
   if not open then exit;
   case usb of
        oVisaCom: get_VisaCom;
        oLibUSB: get_Libusb;
        oRigol: ;
   end;
   result := length(reponse);
end;

function TLIBUSB.readWord(var count : integer;var reponse : CharArray) : integer;
var bufW: PSafeArray;
    i : integer;
    LBound,HBound : integer;
    data : byte;
//    bounds : array[0..0] of TSafeArrayBound;
begin
 //  bounds[0].lLBound := 0;
 //  bounds[0].cElements := count;
 //  bufW := SafeArrayCreate( varByte, 1, @Bounds);
   result := io.Read(count, bufW);
   SafeArrayGetLBound(bufW, 1, LBound);
   SafeArrayGetUBound(bufW, 1, HBound);
   count := HBound-LBound+1;
// S_OK 0
// S_VISA_MAX_CNT 0x40006 : on a lu 1024 caractres
// S_VISA_TERM_CHAR 0x40005 : fin des donnes
   for I := LBound to HBound do
       if SafeArrayGetElement(bufW,i,data)=0 then
          reponse[i-LBound] := data;
   if result<>S_OK then io.Read(count, bufW); // sinon oscillo : query interrupted
//   SafeArrayDestroy(bufW);
{$IFDEF Debug}
      append(fichierDebug);
      writeln(fichierDebug,'#Read:',HBound-LBound+1);
      system.flush(fichierDebug);
      closeFile(fichierDebug);
{$ENDIF}
end;

function TLIBUSB.writeString(str : AnsiString) : boolean;

function send_char(ch : AnsiChar) : boolean;
var z : integer;
begin
    result := usb_control_msg(handle, $C0, 1, ord(ch), 0, z, 0, 1000)=0;
end;

var i : integer;
begin
  result := open;
  if not result then exit;
  case usb of
       oVisaCom : begin
{$IFDEF Debug}
      append(fichierDebug);
      writeln(fichierDebug,timeToStr(time),'Write:',str);
      system.flush(fichierDebug);
      closeFile(fichierDebug);
{$ENDIF}
           i := length(str);
           io.WriteString(wideString(str), i);
           result := i=length(str);
       end;
       oLibUSB : begin
          str := str + crCR;
          for i := 1 to length(str) do
              result := result and
                  send_char(str[i]);
          end;
    end;
end;


procedure TLIBUSB.flush;

var buf : AnsiString;
   len : integer;
begin
   if not open then exit;
   case usb of
        oLibUSB : repeat
           len := readString(buf);
        until (len=0);
        oVisaCom : ; // io.flushRead;
   end;
end;

destructor TLIBUSB.Destroy;
begin
   open := false;
   inherited;
end;

procedure LoadDLL;
begin
  if DLLLoaded then Exit;
  ErrorMode := SetErrorMode($8000); // SEM_NoOpenFileErrorBox
  DLLHandle := LoadLibrary('libusb0.DLL');
  if DLLHandle >= 32 then begin
    DLLLoaded := True;
      //LibUSB functions declarations
    @usb_init := GetProcAddress(DLLHandle,'usb_init');
    Assert(@USB_init <> nil);
    @usb_set_debug := GetProcAddress(DLLHandle,'usb_set_debug');
    Assert(@USB_set_debug <> nil);
    @usb_devices := GetProcAddress(DLLHandle,'usb_device');
    Assert(@USB_devices <> nil);
    @usb_find_busses := GetProcAddress(DLLHandle,'usb_find_busses');
    Assert(@USB_find_busses <> nil);
    @usb_find_devices := GetProcAddress(DLLHandle,'usb_find_devices');
    Assert(@USB_find_devices <> nil);
    @usb_get_busses := GetProcAddress(DLLHandle,'usb_get_busses');
    Assert(@USB_get_busses <> nil);
    @usb_get_version := GetProcAddress(DLLHandle,'usb_get_version');
    Assert(@USB_get_version <> nil);
    @usb_open := GetProcAddress(DLLHandle,'usb_open');
    Assert(@USB_open <> nil);
    @usb_close := GetProcAddress(DLLHandle,'usb_close');
    Assert(@USB_close <> nil);
    @usb_get_string := GetProcAddress(DLLHandle,'usb_get_string');
    Assert(@USB_get_String <> nil);
    @usb_get_string_simple := GetProcAddress(DLLHandle,'usb_get_string_simple');
    Assert(@USB_get_string_simple <> nil);
    @usb_get_descriptor_by_endpoint := GetProcAddress(DLLHandle,'usb_get_descriptor_by_endpoint');
    Assert(@USB_get_descriptor_by_endpoint <> nil);
    @usb_get_descriptor := GetProcAddress(DLLHandle,'usb_get_descriptor');
    Assert(@USB_get_descriptor <> nil);
    @usb_control_msg := GetProcAddress(DLLHandle,'usb_control_msg');
    Assert(@USB_control_msg <> nil);
    @usb_bulk_write := GetProcAddress(DLLHandle,'usb_bulk_write');
    Assert(@USB_bulk_write <> nil);
    @usb_bulk_read := GetProcAddress(DLLHandle,'usb_bulk_read');
    Assert(@USB_bulk_read <> nil);
    @usb_interrupt_write := GetProcAddress(DLLHandle,'usb_interrupt_write');
    Assert(@USB_interrupt_write <> nil);
    @usb_interrupt_read := GetProcAddress(DLLHandle,'usb_interrupt_read');
    Assert(@USB_interrupt_read <> nil);
    @usb_set_configuration := GetProcAddress(DLLHandle,'usb_set_configuration');
    Assert(@USB_set_configuration <> nil);
    @usb_claim_interface := GetProcAddress(DLLHandle,'usb_claim_interface');
    Assert(@USB_claim_interface <> nil);
    @usb_release_interface := GetProcAddress(DLLHandle,'usb_release_interface');
    Assert(@USB_release_interface <> nil);
    @usb_set_altinterface := GetProcAddress(DLLHandle,'usb_set_altinterface');
    Assert(@USB_set_altinterface <> nil);
    @usb_resetep := GetProcAddress(DLLHandle,'usb_resetep');
    Assert(@USB_resetep <> nil);
    @usb_clear_halt := GetProcAddress(DLLHandle,'usb_clear_halt');
    Assert(@USB_clear_halt <> nil);
    @usb_reset := GetProcAddress(DLLHandle,'usb_reset');
    Assert(@USB_reset <> nil);
  end
  else DLLLoaded := False;
  SetErrorMode(ErrorMode);
end; // LoadDLL

initialization
      LoadDLL;
{$IFDEF Debug}
      assignFile(fichierDebug,mesDocsDir+'AgilentDSO.txt');
      rewrite(fichierDebug);
      writeln(fichierDebug,dateToStr(date));
      system.flush(fichierDebug);
      closeFile(fichierDebug);
{$ENDIF}
      ListeVisaCom := TList<TDataVisaCom>.create;
      try
      rm := CoResourceManager.Create; // Create the VISA COM I/O Resource manager
      CoInitializeEx (NIL, COINIT_APARTMENTTHREADED); // Start COM on this thread
      except
  {$IFDEF Debug}
      append(fichierDebug);
      writeln(fichierDebug,'Problme rm.');
      system.flush(fichierDebug);
      closeFile(fichierDebug);
{$ENDIF}

      end;
finalization
     if DLLLoaded then
        FreeLibrary(DLLHandle);
     ListeVisaCom.free;
     rm := nil;
end.



