unit uniteKer;

interface

uses math,SysUtils;

type

   TuniteDeBase = (_metre, _kilogramme, _seconde, _ampere, _kelvin, _mole, _candela);
   EUniteError  = class(Exception);

   Tunite = class
   private
      FnomUnite: string;
      Dimension: array[TuniteDeBase] of shortint;
      Puissance: integer;
      procedure SetNomUnite(const nu: string);
      procedure MulUnite(op: char; U0: Tunite; puiss: shortint);
      procedure PlusUnite(U0: Tunite);
      procedure InverseUnite;
      procedure setErreurUnite;
   public
      nom: string;
      property nomUnite: string Read FnomUnite Write SetNomUnite;
      constructor Create;
      procedure setUniteComplet(const nu: string; p: shortint;
         d1, d2, d3, d4, d5, d6, d7: shortint);
      procedure Init;
      procedure RecopieUnite(U: Tunite);
      procedure Assign(U: Tunite);
      function FormatUnite(Nombre : double) : string;
      function NomAff(puissValeur: shortint): string;
      function formatNombre(Nombre: double): string;
   end;

const
   NbreUniteConnue  = 20;
   CaracPrefixe: TsyscharSet =
      ['a', 'f', 'p', 'n', '', 'm', 'd', 'c', 'h', 'k', 'M', 'G', 'T', 'P', 'E'];
   NomBase: array[TuniteDeBase] of string[3] =
      ('m', 'kg', 's', 'A', 'K', 'mol', 'cd');
   majuscule  = ['a'..'z'];
   minuscule  = ['A'..'Z'];
   chiffre = ['0'..'9'];

type
   CoefUnite  = (zero, yocto, zepto, atto, femto, pico, nano, micro, milli,
      nulle, kilo, mega, giga, tera, peta, exa, zetta, yotta, infini);

var
   UniteConnue:        array[1..NbreUniteConnue] of Tunite;
   UniteNulle, UTemps: Tunite;

const
   puissanceUnite: array[coefUnite] of shortInt =
      (-27, -24, -21, -18, -15, -12, -9, -6, -3,
      0, +3, +6, +9, +12, +15, +18, +21, +24, +27);

implementation

procedure Tunite.InverseUnite;
var
   i: TuniteDeBase;
begin
   Puissance := -puissance;
   for i := _Metre to _Candela do
      dimension[i] := -dimension[i];
end; {InverseUnite}

procedure Tunite.PlusUnite(U0: Tunite);
var
   i: TuniteDeBase;
begin
   Puissance := puissance + U0.puissance;
   for i := _Metre to _Candela do
      dimension[i] := dimension[i] + U0.dimension[i];
end; {PlusUnite}

procedure Tunite.MulUnite(op: char; U0: Tunite; puiss: shortint);
var
   i: TuniteDeBase;
begin
   case op of
      '*':
      begin
         Puissance := U0.Puissance * puiss;
         for i := _Metre to _Candela do
            dimension[i] := U0.dimension[i] * puiss;
      end;
      '/':
      begin
         if (U0.puissance mod puiss <> 0) then
            setErreurUnite;
         Puissance := U0.Puissance div puiss;
         for i := _Metre to _Candela do
         begin
            if (U0.dimension[i] mod puiss <> 0) then
               setErreurUnite;
            dimension[i] := U0.dimension[i] div puiss;
         end;
      end;
   end; { case }
end;{MulUnite}

procedure Tunite.SetNomUnite(const nu: string);
{ Pour les units dont le nomUnite est impos par l'utilisateur }
var
   Ucourant: Tunite;
   Inverse:  boolean;
   LongUnite: integer;

   procedure AnalysePrefixe(var index: integer);
   begin
      if FnomUnite[index] = '/' then
      begin
         Inc(index);
         inverse := True;
      end;
      if (index < LongUnite) and charinset(FnomUnite[index],caracPrefixe) and
         charinset(FnomUnite[succ(index)],majuscule + minuscule + ['']) then  begin
         case FnomUnite[index] of
            'a': Ucourant.puissance  := -18;
            'f': Ucourant.puissance  := -15;
            'p': Ucourant.puissance  := -12;
            'n': Ucourant.puissance  := -9;
            '': Ucourant.puissance := -6;
            'm': Ucourant.puissance  := -3;
            'c': Ucourant.puissance  := -2;
            'd': Ucourant.puissance  := -1;
            'h': Ucourant.puissance  := +2;
            'k': Ucourant.puissance  := +3;
            'M': Ucourant.puissance  := +6;
            'G': Ucourant.puissance  := +9;
            'T': Ucourant.puissance  := +12;
            'P': Ucourant.puissance  := +15;
            'E': Ucourant.puissance  := +18;
         end;
         Inc(index);
      end;
   end;{AnalysePrefixe}

   procedure AnalyseCode(var index: integer);
   var
      trouve: boolean;
      j: integer;
      nomCourant: string;
      indexInit: integer;

      procedure OubliSeparateur;
      var
         j:  integer;
         UU: Tunite;
      begin
         index := indexInit;
         nomCourant := '';
         repeat
            nomCourant := nomCourant + FnomUnite[index];
            Inc(index);
            for j := 1 to NbreUniteConnue do
            begin
               UU := UniteConnue[j];
               trouve := nomCourant = UU.nomUnite;
               if trouve then
               begin
                  Ucourant.PlusUnite(UU);
                  exit;
               end;
            end;
         until (index > LongUnite) or not charinset(FnomUnite[index],Majuscule + Minuscule);
      end; { OubliSeparateur }

   var
      UU: Tunite;
   begin { AnalyseCode }
      nomCourant := '';
      indexInit  := index;
      repeat
         nomCourant := nomCourant + FnomUnite[index];
         Inc(index);
      until (index > longUnite) or not charinset(FnomUnite[index],Majuscule + Minuscule);
      for j := 1 to NbreUniteConnue do
      begin
         UU := UniteConnue[j];
         trouve := nomCourant = UU.nomUnite;
         if trouve then
         begin
            Ucourant.PlusUnite(UU);
            exit;
         end;
      end;
      oubliSeparateur;
      if not Trouve then
         setErreurUnite;
   end; { AnalyseCode }

   procedure AnalyseSuffixe(var index: integer);
   var
      negatif, kilogramme: boolean;
      z: shortint;
      i: TuniteDeBase;
   begin
      if index > longUnite then
      begin
         if inverse then
            Ucourant.inverseUnite;
         exit;
      end;
      if FnomUnite[index] = '^' then
      begin
         Inc(index);
         if (index > LongUnite) then
            setErreurUnite;
      end;
      negatif := False;
      if charinset(FnomUnite[index],['-', '+']) then
      begin
         negatif := FnomUnite[index]='-';
         Inc(index);
         if (index > LongUnite) then
            setErreurUnite;
      end;
      kilogramme := True;
      for i := _Metre to _Candela do
         if i = _kilogramme then
            kilogramme := kilogramme and (Ucourant.dimension[_kilogramme] <> 0)
         else
            kilogramme := kilogramme and (Ucourant.dimension[i] = 0);
      { cas  faire }
      if charinset(FnomUnite[index],chiffre) then  begin
         z := strToInt(FnomUnite[index]);
         if negatif then
            z := -z;
         Inc(index);
      end
      else
         z := 1;
      if inverse then
         z := -z;
      Ucourant.MulUnite('*', Ucourant, z);
   end; { AnalyseSuffixe }

var
   i: TuniteDeBase;
   index, j: integer;
   stValeur: string;
begin { SetNomUnite }
   try { Erreur Unite }
      FnomUnite := nu;
      puissance := 0;
      for i := _Metre to _Candela do
         dimension[i] := 0;
      longUnite := Length(FnomUnite);
      for j := 1 to NbreUniteConnue do
      begin
         Ucourant := UniteConnue[j];
         if FnomUnite = Ucourant.nomUnite then
         begin
            RecopieUnite(Ucourant);
            exit;
         end;
      end;
      index := 1;
      inverse := False;
      if Pos('1E', FNomUnite) = 1 then
      begin
         index := 3;
         while (index < LongUnite) and charinset(FnomUnite[index],Chiffre + ['-', '+']) do
            Inc(index);
         if index > 3 then
         begin
            stValeur  := copy(FnomUnite, 1, pred(index));
            puissance := StrToInt(StValeur);
         end;
         if FnomUnite[index] = ' ' then
            Inc(index);
      end; { 1E+xx }
      Ucourant := Tunite.Create;
      while (index <= LongUnite) do
      begin
         Ucourant.init;
         AnalysePrefixe(index);
         AnalyseCode(index);
         AnalyseSuffixe(index);
         PlusUnite(Ucourant);
         if (index < LongUnite) and (FnomUnite[index] = '.') then
            Inc(index); { sparateur }
      end;
      Ucourant.Free;
   except
      on EUniteError do
      begin
      end;
   end;
end; { SetNomUnite }

function AjoutePuissDix(nom: string; puiss: shortint): string;
var
   dest: string;
begin
   if puiss = 0 then
      ajoutePuissDix := nom
   else  begin
      dest := '10^' + intToStr(puiss) + ' ';
      ajoutePuissDix := dest + nom;
   end;
end;

function Tunite.NomAff(puissValeur: shortint): string;
   { unit dont on connait le dveloppement en puissance }

   function PrefixeConnu(p: shortint): boolean;
   begin
      PrefixeConnu := (p >= -18) and (p <= 18) and ((p mod 3 = 0) or
         (p = -2) or (p = -1) or (p = 2));
   end;

   function AjoutePrefixe(Nom: string; P: shortint): string;
   const
      lettreUnite: TsyscharSet =
         ['A', 'C', 'F', 'H', 'J', 'K', 'L', 'N', 'P', 'S', 'T', 'V', 'W',
         'b', 'c', 'd', 'g', 'l', 'm', 'r', 's'];
   var
      prefixe: char;
   begin
      if nom = '' then   begin
         ajoutePrefixe := AjoutePuissDix('', P);
         exit;
      end;
      if (length(nom) > 1) and charinset(nom[1],caracPrefixe) and
         charinset(nom[2],lettreUnite) then  begin
         case FnomUnite[1] of
            'a': P  := P - 18;
            'f': P  := P - 15;
            'p': P  := P - 12;
            'n': P  := P - 9;
            '': P := P - 6;
            'm': P  := P - 3;
            'c': P  := P - 2;
            'd': P  := P - 1;
            'h': P  := P + 2;
            'k': P  := P + 3;
            'M': P  := P + 6;
            'G': P  := P + 9;
            'T': P  := P + 12;
            'P': P  := P + 15;
            'E': P  := P + 18;
         end;
         nom := copy(nom, 2, length(nom));
      end;
      case P of
         -18: prefixe := 'a';
         -15: prefixe := 'f';
         -12: prefixe := 'p';
         -9: prefixe  := 'n';
         -6: prefixe  := '';
         -3: prefixe  := 'm';
         -2: prefixe  := 'c';
         -1: prefixe  := 'd';
         0:
         begin
            ajoutePrefixe := nom;
            exit;
         end;
         +2: prefixe  := 'h';
         +3: prefixe  := 'k';
         +6: prefixe  := 'M';
         +9: prefixe  := 'G';
         +12: prefixe := 'T';
         +15: prefixe := 'P';
         +18: prefixe := 'E';
         else   begin
            ajoutePrefixe := AjoutePuissDix(nom, P);
            exit;
         end;
      end;
      ajoutePrefixe := prefixe + nom;
   end;

var
   i:  TuniteDeBase;
   OKnulle, OKconnue: boolean;
   j:  integer;
   UU: Tunite;
   NomPositif: string;
   CasDuMetre: boolean;
begin
   if puissValeur = 0 then  begin
      NomAff := FnomUnite;
      exit;
   end;
   OKnulle := True;
   for i := _Metre to _Candela do
      OKnulle := OKnulle and (dimension[i] = 0);
   if OKnulle then
   begin
      NomAff := AjoutePrefixe(FnomUnite, PuissValeur);
      exit;
   end;
   puissValeur := puissance + puissValeur;
   CasDuMetre  := (dimension[_metre] <> 0) and (dimension[_metre] <> 1);
   for i := _KiloGramme to _Candela do
      CasDuMetre := CasDuMetre and (dimension[i] = 0);
   if CasDuMetre and  { 10^-6 m^2 = (mm)^2}
      (puissValeur mod dimension[_metre] = 0) and
      prefixeConnu(puissValeur div dimension[_metre]) then begin
      NomPositif := AjoutePrefixe('m', puissValeur div dimension[_metre]);
      { Pb des units donnes en mL ? }
      NomPositif := NomPositif + intToStr(dimension[_metre]);
      NomAff := NomPositif;
      exit;
   end;
   for j := 1 to NbreUniteConnue do  begin
      UU := UniteConnue[j];
      for i := _Metre to _Candela do begin
         OKconnue := dimension[i] = UU.dimension[i];
         if not OKconnue then
            break;
      end;
      if OKconnue  then  begin
         Result := AjoutePrefixe(UU.nomUnite, puissValeur - UU.puissance);
         exit;
      end;
   end;
   for j := 1 to NbreUniteConnue do
   begin
      UU := UniteConnue[j];
      for i := _Metre to _Candela do  begin
         if i = _Seconde then
            OKconnue := dimension[_seconde] = UU.dimension[_seconde] - 1
         else
            OKconnue := dimension[i] = UU.dimension[i];
         if not OKconnue then
            break;
      end;
      if OKconnue and (UU.nomUnite <> 'Hz') then   begin
         { = ou <> ? }
         Result := AjoutePrefixe(UU.nomUnite, puissValeur - UU.puissance);
         Result := Result + 's-1';
         exit;
      end;
   end;
   NomAff := AjoutePuissDix(FnomUnite, puissValeur);
end; { NomAff }


constructor Tunite.Create;
begin
   Init;
end;

procedure InitUnite;
begin
   UniteConnue[1] := Tunite.Create;
   UniteConnue[1].setUniteComplet('m', 0, 1, 0, 0, 0, 0, 0, 0);
   UniteConnue[2] := Tunite.Create;
   UniteConnue[2].setUniteComplet('A', 0, 0, 0, 0, 1, 0, 0, 0);
   UniteConnue[3] := Tunite.Create;
   UniteConnue[3].setUniteComplet('g', -3, 0, 1, 0, 0, 0, 0, 0);
   UniteConnue[4] := Tunite.Create;
   UniteConnue[4].setUniteComplet('K', 0, 0, 0, 0, 0, 1, 0, 0);
   UniteConnue[5] := Tunite.Create;
   UniteConnue[5].setUniteComplet('mol', 0, 0, 0, 0, 0, 0, 1, 0);
   UniteConnue[6] := Tunite.Create;
   UniteConnue[6].setUniteComplet('cd', 0, 0, 0, 0, 0, 0, 0, 1);
   UniteConnue[7] := Tunite.Create;
   UniteConnue[7].setUniteComplet('H', 0, 2, 1, -2, -2, 0, 0, 0);
   UniteConnue[8] := Tunite.Create;
   UniteConnue[8].setUniteComplet('mol/l', +3, -3, 0, 0, 0, 0, 1, 0);
   UniteConnue[9] := Tunite.Create;
   UniteConnue[9].setUniteComplet('F', 0, -2, -1, 4, 2, 0, 0, 0);
   UniteConnue[10] := Tunite.Create;
   UniteConnue[10].setUniteComplet('N', 0, 1, 1, -2, 0, 0, 0, 0);
   UniteConnue[11] := Tunite.Create;
   UniteConnue[11].setUniteComplet('o', 0, 2, 1, -3, -2, 0, 0, 0);
   UniteConnue[12] := Tunite.Create;
   UniteConnue[12].setUniteComplet('S', 0, -2, -1, 3, 2, 0, 0, 0);
   UniteConnue[13] := Tunite.Create;
   UniteConnue[13].setUniteComplet('T', 0, 0, 1, -2, -1, 0, 0, 0);
   UniteConnue[14] := Tunite.Create;
   UniteConnue[14].setUniteComplet('V', 0, 2, 1, -3, -1, 0, 0, 0);
   UniteConnue[15] := Tunite.Create;
   UniteConnue[15].setUniteComplet('Hz', 0, 0, 0, -1, 0, 0, 0, 0);
   UniteConnue[16] := Tunite.Create;
   UniteConnue[16].setUniteComplet('s', 0, 0, 0, 1, 0, 0, 0, 0);
   UniteConnue[17] := Tunite.Create;
   UniteConnue[17].setUniteComplet('Pa', 0, -1, 1, -2, 0, 0, 0, 0);
   UniteConnue[18] := Tunite.Create;
   UniteConnue[18].setUniteComplet('J', 0, 2, 1, -2, 0, 0, 0, 0);
   UniteConnue[19] := Tunite.Create;
   UniteConnue[19].setUniteComplet('W', 0, 2, 1, -3, 0, 0, 0, 0);
   UniteConnue[20] := Tunite.Create;
   UniteConnue[20].setUniteComplet('C', 0, 0, 0, 1, 1, 0, 0, 0);
   UniteConnue[20] := Tunite.Create;
   UniteConnue[20].setUniteComplet('', 0, 0, 0, 0, 0, 0, 0, 0);

   UniteNulle := Tunite.Create;
   UniteNulle.setUniteComplet('', 0, 0, 0, 0, 0, 0, 0, 0);
end;

procedure Tunite.SetUniteComplet(const nu: string; p: shortint;
   d1, d2, d3, d4, d5, d6, d7: shortint);
begin
   FnomUnite := nu;
   puissance := p;
   Dimension[_metre] := d1;
   Dimension[_kilogramme] := d2;
   Dimension[_seconde] := d3;
   Dimension[_ampere] := d4;
   Dimension[_kelvin] := d5;
   Dimension[_mole] := d6;
   Dimension[_Candela] := d7;
end;

procedure Tunite.setErreurUnite;
begin
   raise EUniteError.Create('Erreur unit');
end;

procedure Tunite.Init;
var
   i: TuniteDeBase;
begin
   FnomUnite := '';
   Puissance := 0;
   for i := _Metre to _Candela do
      Dimension[i] := 0;
   nom := '';
end;

procedure Tunite.RecopieUnite(U: Tunite);
var
   i: TuniteDeBase;
begin
   FnomUnite := U.nomUnite;
   Puissance := U.puissance;
   for i := _Metre to _Candela do
      Dimension[i] := U.dimension[i];
end;

procedure Tunite.Assign(U: Tunite);
begin
   RecopieUnite(U);
   Nom := U.Nom;
end;

function Tunite.FormatNombre(Nombre: double): string;
begin
    Result := FloatToStrF(nombre, ffFixed, 5, 2) + nomAff(0)
end;

Function Tunite.FormatUnite(Nombre : double) : string;
var negatif : boolean;
    tampon : string;
    unite : coefUnite;
    chifAp : integer;
begin
      negatif := nombre<0;
      nombre := abs(nombre);
      if abs(nombre)<1e-12 then begin
         result := '0';
         exit;
      end;
      unite := nulle;
      while ( nombre>=999 ) and (unite<infini) do begin
          nombre := nombre/1000;
          inc(unite)
      end; { nombre<1000 ou infini }
      while ( nombre<0.999 ) and (unite>zero) do begin
          nombre := nombre*1000;
          Dec(unite)
      end; { nombre>=1 ou zero }
      if Nombre<10
          then chifAp := 2
          else if Nombre<100
              then chifAp := 1
              else chifAp := 0;
      tampon := FloatToStrF(nombre,ffFixed,5,chifAp);
      if negatif then tampon := '-' + tampon;
      result := tampon+' '+nomAff(puissanceUnite[unite]);
end;

procedure LibereUnite;
var
   i: integer;
begin
   for i := 1 to NbreUniteConnue do
      UniteConnue[i].Free;
   UniteNulle.Free;
end;

initialization
   InitUnite

finalization
   LibereUnite
end.

