Unit fft;

interface

uses math, sysUtils, classes;

const MaxVecteurFFT = 16384;

type Tfenetre = (Hamming,FlatTop,Blackman);
     indiceVecteurFFT = 0..MaxVecteurFFT;
     vecteurFFT = array[indiceVecteurFFT] of double;
     vecteurIntFFT = array[indiceVecteurFFT] of integer;

var
    fenetre : Tfenetre = Hamming;
    Xtemps : vecteurFFT;

Procedure Fourier(Nbre : integer;var NbreF : integer;var periode : double;var Xfreq : vecteurFFT);

Implementation

var nbreFcourant : integer;

function Puiss2Inf(N : LongInt) : LongInt;
begin
    result := 1;
    while result<N do result := 2*result;
    if result>N then result := result div 2;
end;

function Puiss2Sup(N : LongInt) : LongInt;
begin
    result := Puiss2Inf(N);
    if result<N then result := 2*result;
end;

Function PuissInt(X,N:integer) : integer; { X^N avec N>=0 }
var Y,i : integer;
begin
    Y := 1;
    for i := 1 to N do Y := Y*X;
    PuissInt := Y;
end;

var tbi : vecteurIntFFT;
    wr,wi,kf,xr,xi : vecteurFFT;

procedure fenetrer(nb : integer);
var j : integer;
    xx : double;
    coeffFenetrage : double;
begin
     for j:=0 to pred(nb) do begin
         xx := kf[j]*Xtemps[j]; { fentrage }
         Xtemps[j] := xx;
         xr[tbi[j]] := xx;
// rangement avec permutation des mesures tableau fentres dans xr[]
         xi[j]:=0;
     end;
     try
     case fenetre of
           Hamming : coeffFenetrage := 1.852;
           Blackman : coeffFenetrage := 2.381;
           FlatTop : coeffFenetrage := 4.638;
           else coeffFenetrage := 1.0;
     end;
     for j:=0 to pred(nb) do begin // renormaliser
         xr[j] := xr[j]*coeffFenetrage;
         Xtemps[j] := Xtemps[j]*coeffFenetrage;
     end;
     except
     end;
end;

PROCEDURE tfd(nb : integer);
 { Transformation proprement dite }
var a,i,j,k,k0,nbbloc,tbloc,r,r0,p0,nombre0 : integer;
    zr,zi,c : double;
begin
  nbbloc:=nb;
  tbloc:=1;
  r:=trunc(0.5+Ln(nb)/ln(2));
  FOR i:=1 TO r do begin
          tbloc:=2*tbloc;
          nbbloc:=nbbloc div 2;
          FOR j:=1 TO nbbloc do begin
             nombre0:=(nb div nbbloc -1) div 2;
             r0:=0;
             FOR k0:=0 TO nombre0 do begin
                    k:=k0+(pred(j))*tbloc;
                    p0:=k+tbloc div 2;
                    a:=r0;
                    zr:=xr[p0]*wr[a]-xi[p0]*wi[a];
                    zi:=xr[p0]*wi[a]+xi[p0]*wr[a];
                    xr[p0]:=xr[k]-zr;
                    xi[p0]:=xi[k]-zi;
                    xr[k]:=xr[k]+zr;
                    xi[k]:=xi[k]+zi;
                    r0:=r0+nbbloc;
             end; {k0}
          end; {j}
  end; {i}
  c:=2/nb;
  xr[0]:=xr[0]/2;
  for i:= 0 to pred(nb) do begin
      xr[i]:=c*xr[i];
      xi[i]:=c*xi[i];
  end;
end;

PROCEDURE inverserIndices(nb : integer);
// Table de l'ordre inverse des bits
var i,j,k,r,p0,q,tiroir : integer;
    dpsn : double;
    angle : double;
begin
  if nbreFcourant=nb then exit;
  dpsn:=-2*pi/nb;
  FOR i:=0 TO pred(nb) do begin
         angle := dpsn*i;
         wr[i]:=cos(angle);
         wi[i]:=sin(angle);
// ATTENTION : les indices ne sont pas permuts
// la permutation selon tbi[] sera faite dans le fentrage
         case fenetre of
              Hamming : kf[i]:=0.54-0.46*cos(i*dpsn);
              Blackman : kf[i]:=0.42-0.5*cos(i*dpsn)+0.08*cos(2*i*dpsn);
              FlatTop : kf[i]:=0.2156-0.416*cos(i*dpsn)
                                     +0.2781*cos(2*i*dpsn)
                                     -0.0836*cos(3*i*dpsn)
                                     +0.069*cos(4*i*dpsn);
              else kf[i]:=1.0;
         end;
    end;
    for i:=0 to pred(nb) do tbi[i]:=i;
    r:=trunc(0.5+Ln(nb)/ln(2));
    for i:=0 to pred(nb) do begin
      p0:=r;
      q:=1;
      k:=0;
      for j:=1 to r do begin
           if (i and puissInt(2,pred(p0)))<>0 then k:=k+q;
           q:=2*q;
           dec(p0);
      end;
      if k>i then begin
            tiroir:=tbi[k];
            tbi[k]:=tbi[i];
            tbi[i]:=tiroir;
      end;
    end;
    nbreFcourant := nb;
end; {inverserIndices}

Procedure Fourier(Nbre : integer;var NbreF : integer;var periode : double;var Xfreq : vecteurFFT);
var i : integer;
    coeffZeroPadding : double;
begin
     NbreF := Puiss2Sup(Nbre);
     Periode := Periode/Nbre*NbreF;
     for i := Nbre to pred(NbreF) do Xtemps[i] := 0; // zero padding
     inverserIndices(NbreF);
     fenetrer(NbreF);
     tfd(NbreF);
     coeffZeroPadding := NbreF/Nbre;
     for i := 0 to (NbreF div 2) do
         Xfreq[i]:=sqrt(sqr(xr[i])+sqr(xi[i]))*coeffZeroPadding;
end; // Fourier

initialization
   NbreFcourant := 0;
end.

