unit statcalc;

interface

uses sysUtils,math,graphics,windows,extctrls,uniteker,Vcl.Grids;

Const
   MaxHisto = 32;
   MaxStat = 256;
   MinPointsDist = 10;
// Les bornes vont de 0  MaxHisto et les classes de 1  MaxHisto
// Classe i = entre bornes(pred(i)) et bornes(i)

   LongNombreAxe    = 5;
   NbreGradMin      = 5;
   DimPointVGA      = 4; { dimension du point en pixel VGA }
   couleurCourbe    = clBlack;
   couleurGauss     = clRed;
   WidthNomStat     = 118;
   WidthValeurStat  = 88;
   WidthGridStat    = WidthNomStat+WidthNomStat+2;

type
  indiceStat = 0..MaxStat;
  vecteurStat = array[indiceStat] of double;

  TcalculStatistique = class(Tunite)
      Nbre : integer;
      indexStat : integer;
      Min,Max,Moyenne,sigma,t95,t99 : double;
      Donnees : vecteurStat;
      EcartDist,DebutDist,maxDist : double;
      MoyDist,NbreDist,BornesDist : vecteurStat;
      StatGrid: TStringGrid;
      StatPaint: TPaintBox;
      couleurFond: Tcolor;
      panel:    Tpanel;
      ax,ay,bx,by:  double;  // entier = rel*a+b

    constructor Create;
    destructor Destroy; override;
    Procedure Calcul;
    procedure AjouteValeur(avaleur : double);
    procedure reset;

    procedure DrawStat;
    procedure WindowXY(xr, yr: double;var xi, yi: integer);
    procedure AfficheStat;
  end;

implementation

const
stIncertitudeType = 'Incertitude-type';
stEcartType = 'Ecart-type ';
stMini = 'Mini';
stMaxi = 'Maxi';
stMoyenne = 'Moyenne';
stValeur = 'Valeur ';
stMinimum = 'Minimum';
stMaximum = 'Maximum';
stTaille = 'Taille ';
stEtendue = 'Etendue : ';
stNom = 'Nom';
stSymbole = 'Symbole';
stUnite = 'Unit';

function Dix(x : integer) : double;
Begin
    Dix := Power(10,X)
end;

Function Student95(ddl : integer) : double;
begin
    student95 := ((4.5082/ddl+2.3090)/ddl+2.4033)/ddl+1.960
end;

Function Student99(ddl : integer) : double;
begin
    student99 := ((29.769/ddl+4.0933)/ddl+5.1992)/ddl+2.576
end;

constructor TcalculStatistique.Create;
begin
   inherited create;
   statGrid := nil;
   statPaint := nil;
end;

procedure TcalculStatistique.Calcul;

Procedure CalculStat;
var i : integer;
    ecart : double;
begin
   if (nbre<2) then exit;
   min := Donnees[0];
   max := Donnees[pred(Nbre)];
   moyenne := 0;
   Sigma := 0;
   t99 := 1;
   t95 := 1;
   for i := 0 to pred(Nbre) do begin
       moyenne := moyenne+Donnees[i];
       sigma := sigma+sqr(Donnees[i]);
       if Donnees[i]<Min then Min := Donnees[i];
       if Donnees[i]>Min then Max := Donnees[i];
   end;
   // erreur arrondi       ?
   ecart := (sigma-sqr(moyenne)/Nbre)/pred(Nbre);
   if ecart<(sigma/Nbre/1E8)
     then sigma := 0
     else sigma := sqrt(ecart);
   moyenne := moyenne/Nbre;
   t99 := Sigma*Student99(Nbre-1)/sqrt(Nbre);
   t95 := Sigma*Student95(Nbre-1)/sqrt(Nbre);
end;

Procedure CalculDist;
var i : integer;
    k : integer;
    trouve : boolean;
begin
      if (sigma<1E-6) or (nbre<minPointsDist) then exit;
      if nbre<128 then EcartDist := sigma else ecartDist := sigma/2;
      DebutDist := (Moyenne-ecartDist/2)-ecartDist*(MaxHisto div 2);
      for i:=0 to MaxHisto do begin
          BornesDist[i] := DebutDist+ecartDist*i;
          NbreDist[i] := 0;
          MoyDist[i] := 0;
      end;
   i := 0;
   for k:=0 to pred(Nbre) do begin
// rangement des donnes dans les intervalles
       trouve := Donnees[k]<=BornesDist[i];
       while (i<MaxHisto) and not trouve do begin
           inc(i);
           trouve := Donnees[k]<=BornesDist[i];
       end;
       if trouve then begin
           NbreDist[i] := NbreDist[i] + 1;
           MoyDist[i] := MoyDist[i]+Donnees[k];
       end;
   end;
  
   maxDist := 0;
   for i:=1 to MaxHisto do begin
       if NbreDist[i]>0
           then MoyDist[i] := MoyDist[i]/NbreDist[i]
           else MoyDist[i] := (BornesDist[pred(i)]+BornesDist[i])/2;
       if NbreDist[i]>maxDist then maxDist := round(NbreDist[i]);
   end;
end; // CalculDist

Procedure TriDonnees;
// Tri Shell Meyer-Baudoin p 456 origine vecteur en 0 ?
var increment,k : integer;
    cle : double;
    i,j : integer;
Begin
   increment := 1;
   while (increment<(Nbre div 3)) do increment := succ(3*increment);
   repeat
     for k := 0 to pred(increment) do begin
       	 i := increment+k;
   	     while (i<Nbre) do begin
	  	     cle := Donnees[i];
	  	     j := i-increment;
	  	     while (j>=0) and (donnees[j]>cle) do begin
               Donnees[j+increment] := Donnees[j];
               dec(j,increment);
  		     end;
           Donnees[j+increment] := cle;
	  	     inc(i,increment);
	       end; { i>=nbre }
      end; { for k }
      increment := increment div 3;
   Until increment=0;
end; // triDonnees

begin
     if nbre<2 then exit;
     TriDonnees;
     calculStat;
     calculDist;
end; // Calcul

destructor TcalculStatistique.Destroy;
begin
     inherited destroy;
end;

procedure TcalculStatistique.ajouteValeur(Avaleur : double);
begin
    if isNan(Avaleur) then exit;
    donnees[indexStat] := avaleur;
    inc(indexStat);
    if indexStat>MaxStat then indexStat := 0;
    if nbre<MaxStat then inc(Nbre);
    calcul;
    if (statGrid<>nil) and statGrid.visible then afficheStat;
    if (statPaint<>nil) and statPaint.visible then DrawStat;
end;

procedure TcalculStatistique.reset;
begin
     Nbre := 0;
     indexStat := 0;
end;

procedure Tcalculstatistique.WindowXY(xr, yr: double;var xi, yi: integer);
begin
      xi := round(Ax * xr + Bx);
      yi := round(Ay * yr + By);
end;

procedure TcalculStatistique.DrawStat;
var echelleGauss : double;

procedure ChercheMonde;
var maxX,minX,maxY : double;
    i : integer;
begin
      echelleGauss := Nbre*2*EcartDist/Sigma;
      maxY := MaxDist+1;
      if echelleGauss*0.2>maxY then maxY := echelleGauss*0.2;
      Ay := -(StatPaint.Height / MaxY);
      By := StatPaint.Height;
      i := 1;
      while (i<maxHisto) and (nbreDist[i]=0) do inc(i);
      minX := BornesDist[i-1];
      i := MaxHisto;
      while (i>1) and (nbreDist[i]=0) do dec(i);
      maxX := BornesDist[i];
      Ax := StatPaint.width / (MaxX-MinX);
      Bx := - MinX*ax;
end; // ChercheMonde

procedure TraceGauss;
Const Nth = 30;
      yth : array[0..Nth] of double =
        (0.0022,0.003,0.004,0.00525,0.00675,
         0.00875,0.0112,0.01415,0.01775,0.022,
         0.027,0.0328,0.0395,0.047,0.05545,
         0.06475,0.07485,0.0857,0.0971,0.10895,
         0.121,0.13305,0.14485,0.15615,0.1666,
         0.17605,0.18415,0.1907,0.1955,0.1985,
         0.19945);
var x,y : vecteurStat;
    i : integer;
    h : double;
    x0,y0 : integer;
begin // comparaison avec la loi normale
     h := sigma/10;
     for i := 0 to 2*Nth do x[i] := moyenne+(i-Nth)*h;
     for i := 0 to Nth do begin
        y[i] := yth[i]*echelleGauss;
        y[2*Nth-i] := y[i];
     end;
     WindowXY(x[0], y[0],  x0, y0);
     StatPaint.canvas.moveto(x0,y0);
     if couleurFond=couleurGauss
        then StatPaint.canvas.pen.color := clWhite
        else StatPaint.canvas.pen.color := couleurGauss;
     StatPaint.canvas.pen.style := psDot;
     for i := 1 to 2*Nth do begin
       WindowXY(x[i], y[i],  x0, y0);
       StatPaint.canvas.lineto(x0,y0);
     end;
end; // TraceGauss

procedure TraceCourbe;
  var
    i: integer;
    xi, yi: integer;
    x0, y0, xpred: integer;
  begin
      StatPaint.canvas.Brush.Style := bsBDIAGONAL;
      StatPaint.canvas.Brush.Color := couleurCourbe;
      StatPaint.canvas.pen.Style := psSolid;
      StatPaint.canvas.pen.Color := couleurCourbe;
      StatPaint.canvas.pen.Width := 1;
      WindowXY(BornesDist[0], 0,  x0, y0);
      WindowXY(BornesDist[0], NbreDist[0],  xpred, yi);
      for i := 1 to maxHisto do begin
        WindowXY(BornesDist[i], NbreDist[i],  xi, yi);
        if (NbreDist[i] > 0) and (xi>0) then begin
            if xpred<0 then xpred := 0;
            StatPaint.canvas.rectangle(xpred, yi, xi - 1, y0);
        end;
        xpred := xi + 1;
      end;
      StatPaint.canvas.pen.style := psDashDot;
      WindowXY(moyenne, 0,  x0, y0);
      StatPaint.canvas.moveto(x0,0);
      StatPaint.canvas.lineto(x0,StatPaint.height);
end;// TraceCourbe

begin
if statPaint=nil then exit;
   StatPaint.canvas.brush.style := bsSolid;
   if couleurFond=clBlack then couleurFond := clBtnFace;
   StatPaint.canvas.brush.color := couleurFond;
   StatPaint.canvas.pen.color := couleurFond;
   StatPaint.canvas.pen.Width := 1;
   StatPaint.canvas.pen.style := psSolid;
   StatPaint.canvas.font.Size := 10;
   StatPaint.canvas.font.color := clBlack;
   StatPaint.canvas.rectangle(0,0,statPaint.width,statPaint.height);
   if nbre<minPointsDist
      then StatPaint.canvas.textOut(4,StatPaint.height div 2,'Nombre='+intToStr(nbre)+'<'+intToStr(minPointsDist))
      else if sigma=0 then StatPaint.canvas.textOut(4,StatPaint.height div 2,nom+' constante')
      else begin
        chercheMonde;
        traceCourbe;
        traceGauss;
      end;
end; // graphe.draw

Procedure TcalculStatistique.AfficheStat;
begin
if statGrid=nil then exit;

with StatGrid do begin
    cells[0,0] := stTaille;cells[1,0] := IntToStr(Nbre);
    cells[0,1] := 'Mini';if nbre>2 then cells[1,1] := formatUnite(min) else cells[1,1] := '';
    cells[0,2] := 'Maxi';if nbre>2 then cells[1,2] := formatUnite(max) else cells[1,2] := '';
    cells[0,3] := stMoyenne;if nbre>2 then cells[1,3] := formatUnite(moyenne) else cells[1,3] := '';
//    cells[0,4] := 'ICm 95% mini';cells[1,4] := formatUnite(moyenne-t95);
//    cells[0,5] := 'ICm 95% maxi';cells[1,5] := formatUnite(moyenne+t95);
    cells[0,4] := stEcartType;if nbre>2 then cells[1,4] := formatUnite(sigma) else cells[1,4] := '';
    cells[0,5] := 'U(m,95%)';if nbre>2 then cells[1,5] := formatUnite(t95) else cells[1,5] := '';
    statGrid.update;
end end;

end.


